from kivy.lang import Builder
from kivy.clock import Clock
from kivy.properties import ObjectProperty, ListProperty
from kivy.core.window import Window
from kivygo.uix.popup import BoxPopup
from kivy.animation import Animation
import time, random, os
from camera4kivy import Preview

from pyzbar import pyzbar
from PIL import Image as PILImage


def get_qrcode(path_img, index=0):
	founded = []
	img = PILImage.open(path_img)
	
	for barcode in pyzbar.decode(img):
		data = barcode.data.decode("utf-8")
		if data not in founded:
			founded.append(data)
	
	if not founded or index == None:
		return None

	return founded[index]


Builder.load_string("""


#:import IconInput kivygo.uix.input.IconInput
#:import ButtonEffect kivygo.uix.button.ButtonEffect
#:import ButtonIcon kivygo.uix.icon.ButtonIcon
#:import AnchorLayoutButton kivygo.uix.anchorlayout.AnchorLayoutButton
#:import hex kivy.utils.get_color_from_hex


<PreviewVCamera>:
	v_size: [0, 0]
	v_pos: [0, 0]

<PlantCamera>:
	on_pre_open: camera.connect_camera()
	on_pre_dismiss: camera.disconnect_camera()
	camera: camera
	sc_prop: [0, 0, 0, 0] # w, h, x, y of scanner
	padding: [0, 0, 0, 0]
	box_padding: [0, 0, 0, 0]
	PreviewVCamera:
		id: camera
		aspect_ratio: '9:16'
		on_size: root._update_camera_options()
		on_pos: root._update_camera_options()
		on_kv_post: root._update_camera_options()
		letterbox_color: hex('#96cfea')
	FloatLayout:
		size_hint: [None, None]
		size: [0, 0]
		BoxLayout:
			size_hint: [None, None]
			size: camera.v_size
			pos: camera.v_pos
			w_scann: (self.x +self.width - root.sc_prop[2] - root.sc_prop[0])
			h_top_scann: ( (self.y + self.height) - (root.sc_prop[3] + root.sc_prop[1]) )
			BoxLayout:
				orientation: 'vertical'
				AnchorLayout:
					size_hint_y: None
					height: '40dp'
					anchor_y: 'center'
					canvas.before:
						Color:
							rgba: hex('#fefefe')
						Rectangle:
							pos: self.pos
							size: self.size
					BoxLayout:
						size_hint_y: None
						height: '30dp'
						BoxLayout:
							size_hint: [None, None]
							size: ['110dp', '30dp']
							AnchorLayoutButton:
								radius: [self.width / 1.7] * 4
								size_hint_y: None
								size: ['60dp', '30dp']
								ButtonIcon:
									size: ['25dp', '25dp']
									# source: icon('flash')
									on_release: root.start_scann()
							Label:
								text: 'Auto'
								bold: True
								color: [0, 0, 0, 1]
								halign: 'left'
								valign: 'center'
								text_size: self.size

						Widget:
						AnchorLayoutButton:
							radius: [self.width / 1.7] * 4
							size_hint_y: None
							size: ['80dp', '30dp']
							ButtonIcon:
								size: ['25dp', '25dp']
								# source: icon('settings')
								on_release: root.start_scann()
				
				AnchorLayoutButton:
					size_hint_y: None
					height: '40dp'
					width: self.parent.width
					ButtonIcon:
						size: ['25dp', '25dp']
						# source: icon('return')
						on_release: root.dismiss()

				AnchorLayout:
					anchor_x: 'center'
					anchor_y: 'center'
					Widget:
						id: scanner
						size_hint: [None, None]
						size: ['150dp', '150dp']
						on_size: root.sc_prop = [*self.size, *self.pos]
						on_pos: root.sc_prop = [*self.size, *self.pos]

						canvas:
							Color:
								rgba: hex('#207afe')
							Line:
								points: [self.x, (self.y + dp(40)), self.x, self.y, (self.x + dp(40)), self.y]
								width: dp(1.5)
							Line:
								points: [self.x, (self.y + self.height - dp(40)), self.x, (self.y + self.height), (self.x + dp(40)), (self.y + self.height)]
								width: dp(1.5)
							Line:
								points: [(self.x + self.width), (self.y + self.height - dp(40)), (self.x + self.width), (self.y + self.height), (self.x + self.width - dp(40)), (self.y + self.height)]
								width: dp(1.5)
							Line:
								points: [(self.x + self.width), (self.y + dp(40)), (self.x + self.width), self.y, (self.x + self.width - dp(40)), self.y]
								width: dp(1.5)
				
				AnchorLayout:
					size_hint_y: None
					height: '35dp'
					anchor_y: 'center'
					canvas.before:
						Color:
							rgba: hex('#fefefe')[0:-1] + [0.8]
						Rectangle:
							pos: self.pos
							size: self.size
					BoxLayout:
						size_hint_y: None
						height: '25dp'
						AnchorLayoutButton:
							radius: [self.width / 1.7] * 4
							size_hint_y: None
							size: ['80dp', '25dp']
							ButtonIcon:
								size: ['25dp', '25dp']
								# source: icon('aspect_ratio')
								on_release: root.start_scann()
						Widget:
						AnchorLayoutButton:
							radius: [self.width / 1.7] * 4
							size_hint_y: None
							size: ['80dp', '25dp']
							ButtonIcon:
								size: ['25dp', '25dp']
								# source: icon('dark_theme')
								on_release: root.start_scann()
				AnchorLayout:
					size_hint_y: None
					height: '100dp'
					anchor_y: 'center'
					canvas.before:
						Color:
							rgba: hex('#e4f2ff')
						Rectangle:
							pos: self.pos
							size: self.size
					BoxLayout:
						size_hint_y: None
						height: '70dp'
						Widget:
						AnchorLayoutButton:
							radius: [self.width / 1.7] * 4
							size_hint_y: None
							size: ['100dp', '70dp']
							ButtonIcon:
								size: ['60dp', '60dp']
								# source: icon('img')
								on_release: root.start_scann()
						AnchorLayoutButton:
							radius: [self.width / 1.7] * 4
							size_hint_y: None
							size: ['100dp', '70dp']
							ButtonIcon:
								size: ['70dp', '70dp']
								# source: icon('btn_camera')
								on_release: root.start_scann()
						AnchorLayoutButton:
							radius: [self.width / 1.7] * 4
							size_hint_y: None
							size: ['100dp', '70dp']
							ButtonIcon:
								size: ['60dp', '60dp']
								# source: icon('btn_photos')
								on_release: root.start_scann()
						Widget:


<QRCode>:
	on_pre_open: camera.connect_camera()
	on_pre_dismiss: camera.disconnect_camera()
	camera: camera
	sc_prop: [0, 0, 0, 0] # w, h, x, y of scanner
	padding: [0, 0, 0, 0]
	box_padding: [0, 0, 0, 0]
	PreviewVCamera:
		id: camera
		aspect_ratio: '9:16'
		on_size: root._update_camera_options()
		on_pos: root._update_camera_options()
		on_kv_post: root._update_camera_options()
	FloatLayout:
		size_hint: [None, None]
		size: [0, 0]
		BoxLayout:
			size_hint: [None, None]
			size: camera.v_size
			pos: camera.v_pos
			w_scann: (self.x + self.width - root.sc_prop[2] - root.sc_prop[0])
			h_top_scann: ( (self.y + self.height) - (root.sc_prop[3] + root.sc_prop[1]) )
			canvas.before:
				Color:
					rgba:  [0.1, 0.2, 0.3, 0.4]
				Rectangle:
					pos: [self.x, self.y]
					size: [(self.w_scann or 0), self.height]
				Rectangle:
					pos: [( self.x + self.width - (self.w_scann or 0) ), self.y]
					size: [(self.w_scann or 0), self.height]
				
				Rectangle:
					pos: [( self.x + (self.w_scann or 0) ), ( self.y + self.height - (self.h_top_scann or 0) )]
					size: [(root.sc_prop[0] or 0), (self.h_top_scann or 0)]
				Rectangle:
					pos: [( self.x + (self.w_scann or 0) ), self.y]
					size: [(root.sc_prop[0] or 0), ( (root.sc_prop[3] or 0) - self.y )]

			BoxLayout:
				orientation: 'vertical'
				AnchorLayout:
					anchor_x: 'center'
					anchor_y: 'center'
					Widget:
						id: scanner
						size_hint: [None, None]
						size: ['300dp', '300dp']
						on_size: root.sc_prop = [*self.size, *self.pos]
						on_pos: root.sc_prop = [*self.size, *self.pos]

						canvas:
							Color:
								rgba: hex('#207afe')
							Line:
								points: [self.x, (self.y + dp(60)), self.x, self.y, (self.x + dp(60)), self.y]
								width: dp(3)
							Line:
								points: [self.x, (self.y + self.height - dp(60)), self.x, (self.y + self.height), (self.x + dp(60)), (self.y + self.height)]
								width: dp(3)
							Line:
								points: [(self.x + self.width), (self.y + self.height - dp(60)), (self.x + self.width), (self.y + self.height), (self.x + self.width -dp(60)), (self.y + self.height)]
								width: dp(3)
							Line:
								points: [(self.x + self.width), (self.y + dp(60)), (self.x + self.width), self.y, (self.x + self.width - dp(60)), self.y]
								width: dp(3)

						Icon:
							pos: [(self.parent.x + self.parent.width - self.width - dp(7)), (self.parent.y + dp(7))]
							background_color: [1, 1, 1, 1]
							radius: [self.width / 2] * 4
							size_hint_y: None
							size: ['35dp', '35dp']
							icon_size: ['30dp', '30dp']
							
							# source: icon('resize')
							color: hex('#207afe')
							mipmap: False

							last_touch: 0
							touch_in: False
							on_touch_down:
								if (self.collide_point(*args[1].pos)): self.last_touch = args[1].x
								self.touch_in = True if ( self.collide_point(*args[1].pos) ) else False
							on_touch_up: self.last_touch = 0
							on_touch_move:
								if self.touch_in: scanner.width += (args[1].x - self.last_touch)
								if self.touch_in: scanner.height += (args[1].x - self.last_touch)
								if self.touch_in: self.last_touch = args[1].x
				BoxLayout:
					size_hint_y: None
					height: '70dp'
					padding: ['0dp', '0dp', '0dp', '15dp']
					Widget:
					AnchorLayoutButton:
						background_color: hex('#333333')
						radius: [self.width / 2] * 4
						size_hint_y: None
						size: ['70dp', '70dp']
						ButtonIcon:
							size: ['45dp', '45dp']
							# source: icon('qrcode')
							on_release: root.start_scann()
							color: hex('#313131')
							canvas:
								Color:
									rgba: [1, 1, 1, 1]
								RoundedRectangle:
									pos: self.pos
									size: self.size
									radius: [self.width / 6] * 4
					Widget:
				AnchorLayoutButton:
					size_hint_y: None
					height: '50dp'
					width: self.parent.width
					ButtonIcon:
						size: ['25dp', '25dp']
						# source: icon('return')
						on_release: root.dismiss()

""")


class PreviewVCamera(Preview):
	v_size = ListProperty([0, 0])
	v_pos = ListProperty([0, 0])


class PlantCamera(BoxPopup):

	camera = ObjectProperty(None)
	sc_prop = ListProperty([0, 0, 0, 0])
	started_scan = False
	callback = ObjectProperty(lambda *a: None)

	def __init__(self, *args, **kwargs):
		super().__init__(*args, **kwargs)
		Clock.schedule_once(self.config)

	def config(self, *args):
		self.camera = self.ids.camera
		
		Window.bind(on_flip=self._update_camera_options)

	def _update_camera_options(self, *args):
		self.camera.v_size = self.camera.preview.view_size
		self.camera.v_pos = self.camera.preview.view_pos
	
	def start_scann(self, *args):
		if self.started_scan: return None

		scanner = self.ids.scanner
		last_size = tuple(map(lambda x: x, scanner.size))
		get_over = lambda *a: self.go_back_scan(last_size)

		anim = Animation(size=tuple(map(lambda x: x*1.3, scanner.size)), d=0.5)
		anim.bind(on_complete=get_over)
		anim.start(scanner)
		self.started_scan = True

	def go_back_scan(self, size):
		scanner = self.ids.scanner
		anim = Animation(size=size, d=0.8)
		anim.bind(on_complete=lambda *a: self.end_scann())
		anim.start(scanner)

	def end_scann(self, *args):
		self.started_scan = False
		Clock.schedule_once(self.next_screen, 1)
	
	def next_screen(self, *args):
		self.dismiss()

class QRCode(BoxPopup):

	camera = ObjectProperty(None)
	sc_prop = ListProperty([0, 0, 0, 0])
	started_scan = False
	callback = ObjectProperty(lambda *a: None)

	def __init__(self, *args, **kwargs):
		super().__init__(**kwargs)
		Clock.schedule_once(self.config)
	
	def config(self, *args):
		self.camera = self.ids.camera
		Window.bind(on_flip=self._update_camera_options)
		
	def _update_camera_options(self, *args):
		self.camera.v_size = self.camera.preview.view_size
		self.camera.v_pos = self.camera.preview.view_pos
	
	def read_qrcode(self, *args):
		path = f'_{int(time.time())}_{random.randint(0, int(time.time()))}.png'
		self.camera.export_to_png(path)
		url = get_qrcode(path)
		os.remove(path)
		return url

	def start_scann(self, *args):
		if self.started_scan: return None

		scanner = self.ids.scanner
		last_size = tuple(map(lambda x: x, scanner.size))
		get_over = lambda *a: self.go_back_scan(last_size)

		anim = Animation(size=tuple(map(lambda x: x*1.3, scanner.size)), d=0.5)
		anim.bind(on_complete=get_over)
		anim.start(scanner)
		self.started_scan = True

	def go_back_scan(self, size):
		scanner = self.ids.scanner
		anim = Animation(size=size, d=0.8)
		anim.bind(on_complete=lambda *a: self.end_scann())
		anim.start(scanner)

	def end_scann(self, *args):
		self.started_scan = False
		result = self.read_qrcode()
		
		if result != None:
			self.dismiss()
			self.callback(result)




from kivy.properties import (
	OptionProperty, ObjectProperty,
	BooleanProperty, NumericProperty,
)

from kivy.uix.widget import Widget
from kivy.clock import Clock
from kivy.config import Config
from weakref import ref
from time import time
from kivy.app import App


class ButtonBehavior(Widget):

	state = OptionProperty('normal', options=('normal', 'down'))
	'''
	The state is 'down' only when the button is currently touched/clicked,
	otherwise its 'normal'.
	'''

	last_touch = ObjectProperty(None)
	'''
	Contains the last relevant touch received by the Button. This is
	used in `on_press` and `on_release` to know which touch dispatched the event.
	'''

	min_state_time = NumericProperty(0)
	'''
	The minimum period of time which the widget must remain in the
	`'down'` state. `min_state_time` is a float and defaults to 0.035.
	This value is taken from `kivy.config.Config`.
	'''

	always_release = BooleanProperty(False)
	'''
	This determines whether or not the widget fires an `on_release` event if
	the touch_up is outside the widget.
	'''

	up_elevation = NumericProperty(3)
	"""
	Elevation level when the button is in the up state.
	"""

	down_elevation = NumericProperty(1)
	"""
	Elevation level when the button is in the down state.
	"""

	pressed = BooleanProperty(False)
	"""
	Read only property that represents if the button is pressed or not.
	"""

	do_text_shrink = BooleanProperty(True)
	"""
	When set to `True`, text on a button will shrink when the button is pressed.
	This is done to mimic the effect that the text is going into the screen on press.
	The amount of change in font size can be adjusted using the property `text_shrink_amount`.

	If you have created a custom widget and want to use this effect in a widget. Set the `id`
	value of your text(which is inside the widget) to `label`. The effect will apply to to any
	label with id = 'label'.
	"""

	text_shrink_amount = NumericProperty(1)
	"""
	Amount to decrease the font_size on button press. Will only work if `do_text_shrink` is `True`.
	You can try setting this to a negative value to make the text shrink.
	If `text_shrink_amount = 1` then the font_size will decrease by '1sp'.
	"""

	disabled = BooleanProperty(False)
	"""
	Whether the widget has been disabled or not
	"""

	def on_press(self, *args):
		pass

	def on_release(self, *args):
		pass

	def __init__(self, **kwargs):
		self.register_event_type('on_press')
		self.register_event_type('on_release')
		if 'min_state_time' not in kwargs:
			self.min_state_time = float(Config.get('graphics',
												   'min_state_time'))
		
		super().__init__(**kwargs)
		self.__state_event = None
		self.__touch_time = None
		self.fbind('state', self.cancel_event)
		Clock.schedule_once(self.elevation_setter, 0)

	def elevation_setter(self, *args):
		self.elev = self.up_elevation

	def _do_press(self):
		self.state = 'down'

	def _do_release(self, *args):
		self.state = 'normal'

	def cancel_event(self, *args):
		if self.__state_event:
			self.__state_event.cancel()
			self.__state_event = None

	def on_touch_down(self, touch):
		if touch.is_mouse_scrolling:
			return False
		
		if not self.collide_point(touch.x, touch.y):
			return False
		
		if self in touch.ud:
			return False
	
		if self.collide_point(*touch.pos) and not self.disabled:
			self.pressed = True
			self.elev = self.down_elevation
			self.dispatch("on_press")

			if "label" in self.ids and self.do_text_shrink:
				self.ids.label.font_size -= self.text_shrink_amount
		
		touch.grab(self)
		touch.ud[self] = True
		self.last_touch = touch
		self.__touch_time = time()
		self._do_press()

		if App._running_app:
			self.dispatch('on_press')
		
		return True

	def on_touch_move(self, touch):
		if touch.grab_current is self:
			return True
		
		if super().on_touch_move(touch):
			return True
		
		return (self in touch.ud)

	def on_touch_up(self, touch):
		if touch.grab_current is not self:
			return super().on_touch_up(touch)
		
		assert(self in touch.ud)
		touch.ungrab(self)
		self.last_touch = touch

		if (not self.always_release and
				not self.collide_point(*touch.pos)):
			self._do_release()
			return None

		if self.pressed and not self.disabled:
			self.elev = self.up_elevation
			self.pressed = False
			self.dispatch("on_release")

			if "label" in self.ids and self.do_text_shrink:
				self.ids.label.font_size += self.text_shrink_amount

		touchtime = time() - self.__touch_time
		if touchtime < self.min_state_time:
			self.__state_event = Clock.schedule_once(
				self._do_release, self.min_state_time - touchtime)
		else:
			self._do_release()
		
		if App._running_app:
			self.dispatch('on_release')
		
		return True

	def trigger_action(self, duration=0.1):
		'''Trigger whatever action(s) have been bound to the button by calling
		both the on_press and on_release callbacks.

		This is similar to a quick button press without using any touch events,
		but note that like most kivy code, this is not guaranteed to be safe to
		call from external threads. If needed use `kivy.clock.Clock` to safely
		schedule this function and the resulting callbacks to be called
		from the main thread.

		Duration is the length of the press in seconds. Pass 0 if you want
		the action to happen instantly.
		'''

		self._do_press()
		self.dispatch('on_press')

		def trigger_release(dt):
			self._do_release()
			self.dispatch('on_release')
		
		if not duration:
			trigger_release(0)
		else:
			Clock.schedule_once(trigger_release, duration)


class ToggleButtonBehavior(ButtonBehavior):
	
	__groups = {}

	group = ObjectProperty(None, allownone=True)
	'''
	Group of the button. If `None`, no group will be used (the button will be
	independent). `group` must be a hashable object, like a string.
	Only one button in a group can be in a 'down' state.
	'''

	allow_no_selection = BooleanProperty(True)
	'''
	This specifies whether the widgets in a group allow no selection i.e.
	everything to be deselected.
	'''

	def __init__(self, **kwargs):
		self._previous_group = None
		super().__init__(**kwargs)

	def on_group(self, *largs):
		groups = ToggleButtonBehavior.__groups
		if self._previous_group:
			group = groups[self._previous_group]
			for item in group[:]:
				if item() is self:
					group.remove(item)
					break
		
		group = self._previous_group = self.group
		if group not in groups:
			groups[group] = []
		r = ref(self, ToggleButtonBehavior._clear_groups)
		groups[group].append(r)

	def _release_group(self, current):
		if self.group == None:
			return None
		
		group = self.__groups[self.group]
		for item in group[:]:
			widget = item()
			if widget == None:
				group.remove(item)
			
			if widget is current:
				continue
			
			widget.state = 'normal'

	def _do_press(self):
		if not self.allow_no_selection:
			return None
	
		if self.group and self.state == 'down':
			return None

		self._release_group(self)
		self.state = 'normal' if self.state == 'down' else 'down'

	def _do_release(self, *args):
		pass

	@staticmethod
	def _clear_groups(wk):
		# auto flush the element when the weak reference have been deleted
		groups = ToggleButtonBehavior.__groups
		for group in list(groups.values()):
			if wk in group:
				group.remove(wk)
				break

	@staticmethod
	def get_widgets(groupname):
		'''
		Return a list of the widgets contained in a specific group. If the
		group doesn't exist, an empty list will be returned.
		
		.. note:
			Always release the result of this method! Holding a reference to
			any of these widgets can prevent them from being garbage collected.
			If in doubt, do::
				l = ToggleButtonBehavior.get_widgets('mygroup')
				# do your job
				del l
		
		.. warning:
			It's possible that some widgets that you have previously
			deleted are still in the list. The garbage collector might need
			to release other objects before flushing them.
		'''
		groups = ToggleButtonBehavior.__groups
		if groupname not in groups:
			return []
		return [x() for x in groups[groupname] if x()][:]




# THIS FILE IS GENERATED USING utils.py
# IF YOU'RE ADDING/REMOVING SOME SHADER, DO NOT EDIT THIS FILE DIRECTLY
# INSTEAD DO WHATEVER YOU WANT WITH THE SHADERS AND RUN utils.py

from .base import CustomTransition
from kivy.properties import StringProperty

available_transitions = ["Angular","Bounce","BowTieHorizontal","BowTieVertical","Burn","ButterflyWaveScrawler","CannabisLeaf","Circle","CircleOpen","ColorPhase","ColourDistance","CrazyParametricFun","CrossHatch","CrossWarp","CrossZoom","Cube","DirectionalEasing","DirectionalWarp","DirectionalWipe","Displacement","DoomScreenTransition","Doorway","Dreamy","DreamyZoom","Fade","FadeColor","FadeGrayscale","FilmBurn","FlyEye","GlitchDisplace","GlitchMemories","GridFlip","Heart","Hexagonalize","Kaleidoscope","LeftRight","LinearBlur","Luma","LuminanceMelt","Morph","Mosaic","MultiplyBlend","PageCurl","Perlin","Pinwheel","Pixelize","PolarFunction","PolkaDotsCurtain","Radial","RandomNoise","RandomSquares","Ripple","RotateScaleFade","RotateTransition","SimpleZoom","SquaresWire","Squeeze","StereoViewer","Swap","Swirl","TangentMotionBlur","TopBottom","UndulatingBurnOut","WaterDrop","Wind","WindowBlinds","WindowSlice","WipeDown","WipeLeft","WipeRight","WipeUp","ZoomInCircles",]
__all__ = available_transitions

class Angular(CustomTransition):
    glsl_file = StringProperty("Angular.glsl")



class Bounce(CustomTransition):
    glsl_file = StringProperty("Bounce.glsl")



class BowTieHorizontal(CustomTransition):
    glsl_file = StringProperty("BowTieHorizontal.glsl")



class BowTieVertical(CustomTransition):
    glsl_file = StringProperty("BowTieVertical.glsl")



class Burn(CustomTransition):
    glsl_file = StringProperty("Burn.glsl")



class ButterflyWaveScrawler(CustomTransition):
    glsl_file = StringProperty("ButterflyWaveScrawler.glsl")



class CannabisLeaf(CustomTransition):
    glsl_file = StringProperty("CannabisLeaf.glsl")



class Circle(CustomTransition):
    glsl_file = StringProperty("Circle.glsl")



class CircleOpen(CustomTransition):
    glsl_file = StringProperty("CircleOpen.glsl")



class ColorPhase(CustomTransition):
    glsl_file = StringProperty("ColorPhase.glsl")



class ColourDistance(CustomTransition):
    glsl_file = StringProperty("ColourDistance.glsl")



class CrazyParametricFun(CustomTransition):
    glsl_file = StringProperty("CrazyParametricFun.glsl")



class CrossHatch(CustomTransition):
    glsl_file = StringProperty("CrossHatch.glsl")



class CrossWarp(CustomTransition):
    glsl_file = StringProperty("CrossWarp.glsl")



class CrossZoom(CustomTransition):
    glsl_file = StringProperty("CrossZoom.glsl")



class Cube(CustomTransition):
    glsl_file = StringProperty("Cube.glsl")



class DirectionalEasing(CustomTransition):
    glsl_file = StringProperty("DirectionalEasing.glsl")



class DirectionalWarp(CustomTransition):
    glsl_file = StringProperty("DirectionalWarp.glsl")



class DirectionalWipe(CustomTransition):
    glsl_file = StringProperty("DirectionalWipe.glsl")



class Displacement(CustomTransition):
    glsl_file = StringProperty("Displacement.glsl")



class DoomScreenTransition(CustomTransition):
    glsl_file = StringProperty("DoomScreenTransition.glsl")



class Doorway(CustomTransition):
    glsl_file = StringProperty("Doorway.glsl")



class Dreamy(CustomTransition):
    glsl_file = StringProperty("Dreamy.glsl")



class DreamyZoom(CustomTransition):
    glsl_file = StringProperty("DreamyZoom.glsl")



class Fade(CustomTransition):
    glsl_file = StringProperty("Fade.glsl")



class FadeColor(CustomTransition):
    glsl_file = StringProperty("FadeColor.glsl")



class FadeGrayscale(CustomTransition):
    glsl_file = StringProperty("FadeGrayscale.glsl")



class FilmBurn(CustomTransition):
    glsl_file = StringProperty("FilmBurn.glsl")



class FlyEye(CustomTransition):
    glsl_file = StringProperty("FlyEye.glsl")



class GlitchDisplace(CustomTransition):
    glsl_file = StringProperty("GlitchDisplace.glsl")



class GlitchMemories(CustomTransition):
    glsl_file = StringProperty("GlitchMemories.glsl")



class GridFlip(CustomTransition):
    glsl_file = StringProperty("GridFlip.glsl")



class Heart(CustomTransition):
    glsl_file = StringProperty("Heart.glsl")



class Hexagonalize(CustomTransition):
    glsl_file = StringProperty("Hexagonalize.glsl")



class Kaleidoscope(CustomTransition):
    glsl_file = StringProperty("Kaleidoscope.glsl")



class LeftRight(CustomTransition):
    glsl_file = StringProperty("LeftRight.glsl")



class LinearBlur(CustomTransition):
    glsl_file = StringProperty("LinearBlur.glsl")



class Luma(CustomTransition):
    glsl_file = StringProperty("Luma.glsl")



class LuminanceMelt(CustomTransition):
    glsl_file = StringProperty("LuminanceMelt.glsl")



class Morph(CustomTransition):
    glsl_file = StringProperty("Morph.glsl")



class Mosaic(CustomTransition):
    glsl_file = StringProperty("Mosaic.glsl")



class MultiplyBlend(CustomTransition):
    glsl_file = StringProperty("MultiplyBlend.glsl")



class PageCurl(CustomTransition):
    glsl_file = StringProperty("PageCurl.glsl")



class Perlin(CustomTransition):
    glsl_file = StringProperty("Perlin.glsl")



class Pinwheel(CustomTransition):
    glsl_file = StringProperty("Pinwheel.glsl")



class Pixelize(CustomTransition):
    glsl_file = StringProperty("Pixelize.glsl")



class PolarFunction(CustomTransition):
    glsl_file = StringProperty("PolarFunction.glsl")



class PolkaDotsCurtain(CustomTransition):
    glsl_file = StringProperty("PolkaDotsCurtain.glsl")



class Radial(CustomTransition):
    glsl_file = StringProperty("Radial.glsl")



class RandomNoise(CustomTransition):
    glsl_file = StringProperty("RandomNoise.glsl")



class RandomSquares(CustomTransition):
    glsl_file = StringProperty("RandomSquares.glsl")



class Ripple(CustomTransition):
    glsl_file = StringProperty("Ripple.glsl")



class RotateScaleFade(CustomTransition):
    glsl_file = StringProperty("RotateScaleFade.glsl")



class RotateTransition(CustomTransition):
    glsl_file = StringProperty("RotateTransition.glsl")



class SimpleZoom(CustomTransition):
    glsl_file = StringProperty("SimpleZoom.glsl")



class SquaresWire(CustomTransition):
    glsl_file = StringProperty("SquaresWire.glsl")



class Squeeze(CustomTransition):
    glsl_file = StringProperty("Squeeze.glsl")



class StereoViewer(CustomTransition):
    glsl_file = StringProperty("StereoViewer.glsl")



class Swap(CustomTransition):
    glsl_file = StringProperty("Swap.glsl")



class Swirl(CustomTransition):
    glsl_file = StringProperty("Swirl.glsl")



class TangentMotionBlur(CustomTransition):
    glsl_file = StringProperty("TangentMotionBlur.glsl")



class TopBottom(CustomTransition):
    glsl_file = StringProperty("TopBottom.glsl")



class UndulatingBurnOut(CustomTransition):
    glsl_file = StringProperty("UndulatingBurnOut.glsl")



class WaterDrop(CustomTransition):
    glsl_file = StringProperty("WaterDrop.glsl")



class Wind(CustomTransition):
    glsl_file = StringProperty("Wind.glsl")



class WindowBlinds(CustomTransition):
    glsl_file = StringProperty("WindowBlinds.glsl")



class WindowSlice(CustomTransition):
    glsl_file = StringProperty("WindowSlice.glsl")



class WipeDown(CustomTransition):
    glsl_file = StringProperty("WipeDown.glsl")



class WipeLeft(CustomTransition):
    glsl_file = StringProperty("WipeLeft.glsl")



class WipeRight(CustomTransition):
    glsl_file = StringProperty("WipeRight.glsl")



class WipeUp(CustomTransition):
    glsl_file = StringProperty("WipeUp.glsl")



class ZoomInCircles(CustomTransition):
    glsl_file = StringProperty("ZoomInCircles.glsl")




from kivy.properties import StringProperty, BoundedNumericProperty
from kivy.uix.relativelayout import RelativeLayout
from kivy.graphics import Color, Ellipse, Mesh, Scale
from kivy.utils import get_color_from_hex


class Segment(RelativeLayout):
  
    scale = BoundedNumericProperty(0.1, min=0.1, max=1, errorvalue=0.2)
    color = StringProperty('2fc827')
    value = StringProperty('A.')

    def __init__(self, **kwargs):     
        super().__init__(**kwargs)

        # Drawing meshes configuration, indices range meshes and mode
        self.indice = range(0, 6)
        self.xmode = 'triangle_fan'
        
        # Segment matrix configuration
        seg_1 = [
            20, 215, 0, 0,
            35, 230, 0, 0,
            95, 230, 0, 0,
            110, 215, 0, 0,
            95, 200, 0, 0,
            35, 200, 0, 0,
        ]
        seg_2 = [
            15, 210, 0, 0,
            30, 195, 0, 0,
            30, 135, 0, 0,
            15, 120, 0, 0,
            0, 135, 0, 0,
            0, 195, 0, 0,
        ]
        seg_3 = [
            115, 210, 0, 0,
            130, 195, 0, 0,
            130, 135, 0, 0,
            115, 120, 0, 0,
            100, 135, 0, 0,
            100, 195, 0, 0,
        ]
        seg_4 = [
            20, 115, 0, 0,
            35, 130, 0, 0,
            95, 130, 0, 0,
            110, 115, 0, 0,
            95, 100, 0, 0,
            35, 100, 0, 0,
        ]
        seg_5 = [
            15, 110, 0, 0,
            30, 95, 0, 0,
            30, 35, 0, 0,
            15, 20, 0, 0,
            0, 35, 0, 0,
            0, 95, 0, 0,
        ]
        seg_6 = [
            115, 110, 0, 0,
            130, 95, 0, 0,
            130, 35, 0, 0,
            115, 20, 0, 0,
            100, 35, 0, 0,
            100, 95, 0, 0,
        ]
        seg_7 = [
            20, 15, 0, 0,
            35, 30, 0, 0,
            95, 30, 0, 0,
            110, 15, 0, 0,
            95, 0, 0, 0,
            35, 0, 0, 0,
        ]

        # Drawing association
        type_0 = [seg_1, seg_2, seg_3, seg_5, seg_6, seg_7]
        type_1 = [seg_3, seg_6]
        type_2 = [seg_1, seg_3, seg_4, seg_5, seg_7]
        type_3 = [seg_1, seg_3, seg_4, seg_6, seg_7]
        type_4 = [seg_2, seg_3, seg_4, seg_6]
        type_5 = [seg_1, seg_2, seg_4, seg_6, seg_7]
        type_6 = [seg_1, seg_2, seg_4, seg_5, seg_6, seg_7]
        type_7 = [seg_1, seg_3, seg_6]
        type_8 = [seg_1, seg_2, seg_3, seg_4, seg_5, seg_6, seg_7]
        type_9 = [seg_1, seg_2, seg_3, seg_4, seg_6, seg_7]
        type_A = [seg_1, seg_2, seg_3, seg_4, seg_5, seg_6]
        type_b = [seg_2, seg_4, seg_5, seg_6, seg_7]
        type_C = [seg_1, seg_2, seg_5, seg_7]
        type_d = [seg_3, seg_4, seg_5, seg_6, seg_7]
        type_E = [seg_1, seg_2, seg_4, seg_5, seg_7]
        type_F = [seg_1, seg_2, seg_4, seg_5]

        # Routing association
        self.type_dic = {
                "0" : type_0,
                "0.": type_0,
                "1" : type_1,
                "1.": type_1,
                "2" : type_2,
                "2.": type_2,
                "3" : type_3,
                "3.": type_3,
                "4" : type_4,
                "4.": type_4,
                "5" : type_5,
                "5.": type_5,
                "6" : type_6,
                "6.": type_6,
                "7" : type_7,
                "7.": type_7,
                "8" : type_8,
                "8.": type_8,
                "9" : type_9,
                "9.": type_9,
                "A" : type_A,
                "A.": type_A,
                "b" : type_b,
                "b.": type_b,
                "C" : type_C,
                "C.": type_C,
                "d" : type_d,
                "d.": type_d,
                "E" : type_E,
                "E.": type_E,
                "F" : type_F,
                "F.": type_F,
                }

        # Binding refresh drawing method
        self.bind(
            pos=self._update_canvas, 
            size=self._update_canvas,
            value=self._update_canvas
            )

    def _update_canvas(self, *args):

        with self.canvas:

            # Refresh
            self.canvas.clear()

            # Configure
            Color(
                get_color_from_hex(self.color)[0], 
                get_color_from_hex(self.color)[1], 
                get_color_from_hex(self.color)[2], 100
            )

            Scale(self.scale)

            def make_mesh(self, ttype, *args):
                for segment in ttype:
                    Mesh(
                        vertices=segment, 
                        indices=self.indice, 
                        mode=self.xmode
                        )
                if len(self.value) > 1:
                    Ellipse(pos=(135, 0), size=(25,25), segments=360)

            # Avoid if session
            for key, val in self.type_dic.items():
                if self.value == key:
                    make_mesh(self, ttype=val)
      




from kivy.uix.anchorlayout import AnchorLayout
from kivy.uix.label import Label
from kivy.clock import Clock
from kivy.graphics import Line, Rectangle, Color, Ellipse
from kivy.properties import (
	OptionProperty, BoundedNumericProperty,
	ColorProperty, NumericProperty,
	ObjectProperty, AliasProperty,
	ListProperty, StringProperty,
	BooleanProperty
)

from kivy.metrics import dp
from kivy.lang.builder import Builder
from math import ceil


Builder.load_string("""

<CircularProgressBar>:
	anchor_x: "center"
	anchor_y: "center"
	Widget:
		id: circular_widget
		size_hint: None, None
		size: [root.widget_size] * 2

""")


class CircularProgressBar(AnchorLayout):

	started  = BooleanProperty(False)

	# This constant enforces the cap argument to be one of the caps accepted by the kivy.graphics.Line class
	cap_style = OptionProperty("round", options=["round", "none", "square"])
	""" cap / edge of the bar, check the cap keyword argument in kivy.graphics.Line
	"""

	border_width = BoundedNumericProperty(dp(10), min=dp(1))
	""" border_width of the progress bar line (positive integer)
	"""

	cap_precision = BoundedNumericProperty(10, min=1)
	""" bar car sharpness, check the cap_precision keyword argument in kivy.graphics.Line
	"""

	widget_size = BoundedNumericProperty(dp(200), min=dp(1))
	""" size of the widget, use this to avoid issues with size, width, height etc.
	"""

	progress_color = ColorProperty([1, 0, 0, 1])
	""" Colour progress of the progress bar, check values accepted by kivy.graphics.Color
	"""

	progress_background = ColorProperty([0.26, 0.26, 0.26, 1])
	""" Colour progress of the background bar, check values accepted by kivy.graphics.Color
	"""

	background_color = ColorProperty("#d4c9d0")
	"""
	"""

	_progress = NumericProperty(0)

	def _get_progress(self):
		return self._progress

	def _set_progress(self, progress):
		if progress < 0:
			self._progress = self.min_progress

		elif progress > self.max_progress:
			self._progress = self.max_progress
			# raise ValueError(f"Progress must be between minimum ({self.min_progress}) and maximum ({self.max_progress}), not {self._progress}!")
		else:
			self._progress = progress

	progress = AliasProperty(_get_progress, _set_progress, bind=["_progress"])
	""" progress progress, can you use it initialise the bar to some other progress different from the minimum
	"""

	# Declare the defaults for the normalisation function, these are used in the textual representation (multiplied by 100)
	normalised_max = NumericProperty(1)
	normalised_min = NumericProperty(0)

	max_progress = NumericProperty(100)
	""" maximum progress (progress corresponding to 100%)
	"""

	min_progress = NumericProperty(0)
	""" minimum progress (progress corresponding to 0%) - note that this sets the starting progress to this progress
	"""

	label = ObjectProperty(None)
	"""
		kivy.graphics.Label textually representing the progress - pass a label with an empty text field to remove it,
		use "{}" as the progress progress placeholder (it will be replaced via the format function)
	"""

	label_size = ListProperty([0, 0])
	label_args = ListProperty([])

	label_text = StringProperty("")
	label_color = ColorProperty([0.1, 0.4, 0, 1])

	def get_normalised_progress(self):
		"""
		Function used to normalise the progress using the MIN/MAX normalisation

		:return: Current progress normalised to match the percentage constants
		"""
		dif1 = (self.normalised_max - self.normalised_min)
		dif2 = (self.max_progress - self.min_progress)
		return self.normalised_min + ((self._progress - self.min_progress) * (dif1 / dif2))

	def set_normalised_progress(self, norm_progress):
		"""
		Function used to set the progress progress from a normalised progress, using MIN/MAX normalisation

		:param norm_progress: Normalised progress to update the progress with
		"""
		if not isinstance(norm_progress, (float, int)):
			raise TypeError("Normalised progress must be a float or an integer, not {}!".format(
				type(norm_progress)))

		# if norm_progress < self.normalised_min or norm_progress > self.normalised_max:
		# 	return None

		dif1 = (self.max_progress - self.min_progress)
		dif2 = (self.normalised_max - self.normalised_min)
		self._progress = ceil(
			self.min_progress + (norm_progress - self.normalised_min) * dif1 / dif2)

	value_normalized = AliasProperty(
		get_normalised_progress, set_normalised_progress, bind=["_progress"])

	def on_label(self, *args):
		if not self.label_text:
			self.label_text = self.label.text

	def __init__(self, **kwargs):

		self.label = Label(text="Circular Bar", font_size=19, bold=True)
		self._progress = self.min_progress
		super().__init__(**kwargs)

		self.bind(label=self._draw)
		self.bind(progress_background=self._draw)
		self.bind(background_color=self._draw)
		self.bind(border_width=self._draw)
		self.bind(cap_style=self._draw)
		self.bind(label_text=self._draw)
		self.bind(label_size=self._draw)
		self.bind(_progress=self._draw)
		self.bind(label_color=self._draw)

		Clock.schedule_once(self.start)

	def on_kv_post(self, *args):
		self.ids.circular_widget.bind(size=self.start)
	
	def start(self, *args):
		if self.get_root_window() != None:
			self.ids.circular_widget.unbind(size=self.start)
			if not self.started:
				Clock.schedule_once(self.start)
				self.started = True
				return None
		else:
			return None
	
		self._draw()


	def on_max_progress(self, *args):
		if self.max_progress <= self.min_progress:
			raise ValueError(
				f"Maximum progress - {self.max_progress} - must be greater than minimum progress ({self.min_progress})!")

		if self._progress > self.max_progress:
			self._progress = self.max_progress

	def on_min_progress(self, *args):
		if self.min_progress > self.max_progress:
			raise ValueError(
				f"Minimum progress - {self.min_progress} - must be smaller than maximum progress ({self.max_progress})!")

		if self._progress < self.min_progress:
			self._progress = self.min_progress

	def _refresh_text(self, *args):
		"""
		Function used to refresh the text of the progress label.

		Additionally updates the variable tracking the label's texture size
		"""
		self.label.color = self.label_color

		if self.label_args:
			self.label.text = self.label_text.format(*self.label_args)
		else:
			self.label.text = self.label_text

		self.label_size = self.label.texture_size

	def _draw(self, *args):
		"""
		Function used to draw the progress bar onto the screen.

		The drawing process is as follows:

				1. Clear the canvas
				2. Draw the background_color progress line (360 degrees)
				3. Draw the actual progress line (N degrees where n is between 0 and 360)
				4. Draw the textual representation of progress in the middle of the circle
		"""
		wid = self.ids.circular_widget

		with wid.canvas:
			wid.canvas.clear()
			self._refresh_text()

			# Draw the progress line
			Color(*self.background_color)
			Ellipse(pos=list(map(lambda v: (v + self.border_width), wid.pos)),
					size=list(map(lambda v: (v - (self.border_width * 2)), wid.size)))

			# Draw the background progress line
			Color(*self.progress_background)
			Line(ellipse=(*wid.pos, *wid.size), width=self.border_width)

			# Draw the progress line
			Color(*self.progress_color)
			Line(ellipse=(*wid.pos, *wid.size, 0, self.value_normalized * 360),
				 width=self.border_width, cap=self.cap_style, cap_precision=self.cap_precision)

			# Center and draw the progress text
			Color(1, 1, 1, 1)
			Rectangle(texture=self.label.texture, size=self.label_size,
					  pos=(wid.center_x - (self.label_size[0] / 2),
						   wid.center_y - (self.label_size[1] / 2)))




from kivygo.behaviors.button import ButtonBehavior, ToggleButtonBehavior
from kivygo.behaviors.touch_effecs import EffectBehavior
from kivygo.behaviors.hover import HoverBehavior

from kivygo.uix.image import ImageWithSVG
from kivygo.uix.anchorlayout import AnchorLayoutButton
from kivy.properties import ListProperty, BooleanProperty, StringProperty
from kivy.metrics import dp
from kivy.lang import Builder
from kivy.clock import Clock


Builder.load_string("""

#:import ImageWithSVG kivygo.uix.image.ImageWithSVG

<Icon>:
	anchor_y: 'center'
	anchor_x: 'center'

	BoxLayout:
		padding: ['5dp', '5dp', '5dp', '5dp']
		size_hint: [None, None]
		size: root.icon_size

		ImageWithSVG:
			image_source: root.icon_source
			allow_stretch: root.allow_stretch 
			keep_ratio: root.keep_ratio
			mipmap: root.mipmap
			color: root.color


<ButtonIcon>:
	size_hint: [None, None]
	mipmap: True
	allow_strech: True
	keep_ratio: False
	canvas:
		Clear
	canvas.before:
		Color:
			rgba: self._background_color
		RoundedRectangle:
			pos: self.pos
			size: self.size
			radius: self.radius
	canvas.after:
		Color:
			rgba: self.color
		Rectangle:
			texture: self.texture
			pos: self.pos
			size: self.size


<ToggleButtonIcon>:
	size: ['30dp', '30dp']


""")


class Icon(AnchorLayoutButton):

	color = ListProperty([1, 1, 1, 1])
	allow_stretch = BooleanProperty(True)
	keep_ratio = BooleanProperty(False)
	mipmap = BooleanProperty(True)
	icon_source = StringProperty("")
	icon_size = ListProperty([dp(40), dp(40)])


class ButtonIcon(ButtonBehavior, ImageWithSVG, EffectBehavior, HoverBehavior):

	# Properties
	icon_state_color = ListProperty([-1, -1, -1, -1])
	icon_color = ListProperty([1, 1, 1, 1])
	icon_state_source = ListProperty(['', ''])
	background_color = ListProperty([-1, -1, -1, -1])
	_background_color = ListProperty([0, 0, 0, 0])
	radius = ListProperty([0, 0, 0, 0])
	
	def __init__(self, **kwargs):
		self.type_button = 'rounded'
		self.color = [1, 1, 1, 1]
		super().__init__(**kwargs)
		Clock.schedule_once(self.set_color)
	
	def set_color(self, *args):

		if self.image_source != '' and len(self.image_source) != 2:
			self.set_source(self.image_source)

		elif len(self.image_source) == 2 and '' not in self.image_source:
			self.set_source(self.image_source[0])

		elif len(self.icon_state_source) == 2 and '' not in self.icon_state_source:
			self.set_source(self.icon_state_source[0])
		
		if isinstance(self.icon_color[0], (int, float)):
			self.color = self.icon_color
		else:
			self.color = self.icon_color[0]
		
		if isinstance(self.icon_state_color[0], (int, float)):

			if self.icon_state_color != [-1, -1, -1, -1]:
				self.color = self.icon_state_color
		else:
			self.color = self.icon_state_color[0]
		
		if isinstance(self.background_color[0], (int, float)):

			if self.background_color != [-1, -1, -1, -1]:
				self._background_color = self.background_color
		else:
			self._background_color = self.background_color[0]

		Clock.schedule_once(lambda *a: self.on_state(self, self.state))
	

	def on_state(self, widget, state):
		
		if len(self.icon_state_source) == 2 and '' not in self.icon_state_source:

			if state == 'normal':
				self.set_source(self.icon_state_source[0])

			elif state == 'down':
				self.set_source(self.icon_state_source[1])

		if state == "down":

			if not isinstance(self.icon_state_color[0], (int, float)):
				self.color = self.icon_state_color[1]

			if not isinstance(self.background_color[0], (int, float)):
				self._background_color = self.background_color[1]
			
			return None
		
		if not isinstance(self.icon_state_color[0], (int, float)):
			self.color = self.icon_state_color[0]

		if not isinstance(self.background_color[0], (int, float)):

			self._background_color = self.background_color[0]


	def on_cursor_enter(self, *args):

		if len(self.image_source) == 2 and '' not in self.image_source:
			self.set_source(self.image_source[1])

		if not isinstance(self.icon_color[0], (int, float)):
			self.color = self.icon_color[1]

		return super().on_cursor_enter(*args)

	def on_cursor_leave(self, *args):
		
		if len(self.image_source) == 2 and '' not in self.image_source:
			self.set_source(self.image_source[0])
		
		if not isinstance(self.icon_color[0], (int, float)):
			self.color = self.icon_color[0]
			
		return super().on_cursor_leave(*args)

	def on_touch_down(self, touch):
		v = super().on_touch_down(touch)
		if not self.collide_point(*touch.pos):
			return False
		
		self.ripple_show(touch)
		return v
		
	def on_touch_up(self, touch):
		v = super().on_touch_up(touch)
		if not self.collide_point(*touch.pos):
			return False
		
		self.ripple_fade()
		return v


class ToggleButtonIcon(ToggleButtonBehavior, ButtonIcon):
	pass





from kivy.animation import Animation
from kivy.metrics import dp
from kivy.graphics import Color, Ellipse, Rectangle
from kivy.event import EventDispatcher
from kivy.properties import (
	ObjectProperty, NumericProperty,
	ListProperty, StringProperty,
	BooleanProperty, OptionProperty
)
from kivy.uix.label import Label


class TapTargetView(EventDispatcher):
	"""Rough try to mimic the working of Android's TapTargetView."""

	widget = ObjectProperty(None)
	"""
	Widget to add ``TapTargetView`` upon.
	"""

	outer_radius = NumericProperty(dp(300))
	"""
	Radius for outer circle.
	"""

	outer_circle_color = ListProperty([1, 0, 0])
	"""
	Color for the outer circle.
	"""

	outer_circle_alpha = NumericProperty(0.96)
	"""
	Alpha value for outer circle.
	"""

	target_radius = NumericProperty(dp(45))
	"""
	Radius for target circle.
	"""

	target_circle_color = ListProperty([1, 1, 1])
	"""
	Color for target circle.
	"""

	title_text = StringProperty("")
	"""
	Title to be shown on the view.
	"""

	title_text_size = NumericProperty(dp(25))
	"""
	Text size for title.
	"""

	title_text_color = ListProperty([1, 1, 1, 1])
	"""
	Text color for title.
	"""

	title_text_bold = BooleanProperty(True)
	"""
	Whether title should be bold.
	"""

	description_text = StringProperty("")
	"""
	Description to be shown below the title (keep it short).
	"""

	description_text_size = NumericProperty(dp(20))
	"""
	Text size for description text.
	"""

	description_text_color = ListProperty([0.9, 0.9, 0.9, 1])
	"""
	Text size for description text.
	"""

	description_text_bold = BooleanProperty(False)
	"""
	Whether description should be bold.
	"""

	draw_shadow = BooleanProperty(False)
	"""
	Whether to show shadow.
	"""

	cancelable = BooleanProperty(False)
	"""
	Whether clicking outside the outer circle dismisses the view.
	"""

	widget_position = OptionProperty(
		"left",
		options=[
			"left", "right",
			"top", "bottom",
			"left_top", "right_top",
			"left_bottom", "right_bottom",
			"center",
		],
	)
	"""
	Sets the position of the widget on the outer_circle. Available options are
	"""

	title_position = OptionProperty(
		"auto",
		options=[
			"auto",
			"left", "right",
			"top", "bottom",
			"left_top", "right_top",
			"left_bottom", "right_bottom",
		],
	)
	"""
	Sets the position of :attr`~title_text` on the outer circle. Only works if
	:attr`~widget_position` is set to `'center'`. In all other cases, it
	calculates the :attr`~title_position` itself.
	Must be set to other than `'auto`' when :attr`~widget_position` is set
	to `'center`'.
	"""

	stop_on_outer_touch = BooleanProperty(False)
	"""
	Whether clicking on outer circle stops the animation.
	"""

	stop_on_target_touch = BooleanProperty(True)
	"""
	Whether clicking on target circle should stop the animation.
	"""

	end = ObjectProperty(None)
	"""
	Function to be called when the animation stops.
	"""


	def on_description_text(self, instance, value):
		self.core_description_text.text = value

	def on_description_text_size(self, instance, value):
		self.core_description_text.font_size = value

	def on_description_text_bold(self, instance, value):
		self.core_description_text.bold = value

	def on_title_text(self, instance, value):
		self.core_title_text.text = value

	def on_title_text_size(self, instance, value):
		self.core_title_text.font_size = value

	def on_title_text_bold(self, instance, value):
		self.core_title_text.bold = value

	def on_target_touch(self):
		if self.stop_on_target_touch:
			self.stop()

	def on_outer_touch(self):
		if self.stop_on_outer_touch:
			self.stop()

	def on_outside_click(self):
		if self.cancelable:
			self.stop()


	def __init__(self, **kwargs):
		self.ripple_max_dist = dp(90)
		self.outer_radius *= 2
		self.target_radius *= 2

		self.core_title_text = Label(
			markup=True, size_hint=(None, None), bold=self.title_text_bold
		)

		self.core_title_text.bind(texture_size=self.core_title_text.setter("size"))
		self.core_description_text = Label(markup=True, size_hint=(None, None))
		
		self.core_description_text.bind(
			texture_size=self.core_description_text.setter("size")
		)

		super().__init__(**kwargs)
		self.register_event_type("on_outer_touch")
		self.register_event_type("on_target_touch")
		self.register_event_type("on_outside_click")

	def _initialize(self):
		setattr(self.widget, "outer_radius", 0)
		setattr(self.widget, "target_radius", 0)
		setattr(self.widget, "target_ripple_radius", 0)
		setattr(self.widget, "target_ripple_alpha", 0)

		# Bind some function on widget event when this function is called
		# instead of when the class itself is initialized to prevent all
		# widgets of all instances to get bind at once and start messing up.
		self.widget.bind(on_touch_down=self._some_func)

	def _draw_canvas(self):
		_pos = self._ttv_pos()
		self.widget.canvas.before.clear()

		with self.widget.canvas.before:
			# Outer circle.
			Color(*self.outer_circle_color, self.outer_circle_alpha, group="ttv_group")
			_rad1 = self.widget.outer_radius
			Ellipse(size=[_rad1, _rad1], pos=_pos[0], group="ttv_group")

			# Title text.
			Color(*self.title_text_color, group="ttv_group")
			Rectangle(
				size=self.core_title_text.texture.size,
				texture=self.core_title_text.texture,
				pos=_pos[1],
				group="ttv_group",
			)

			# Description text.
			Color(*self.description_text_color, group="ttv_group")
			Rectangle(
				size=self.core_description_text.texture.size,
				texture=self.core_description_text.texture,
				pos=[_pos[1][0], _pos[1][1] - self.core_description_text.size[1] - 5],
				group="ttv_group",
			)

			# Target circle.
			Color(*self.target_circle_color, group="ttv_group")
			_rad2 = self.widget.target_radius
			Ellipse(
				size=[_rad2, _rad2],
				pos=[
					self.widget.x - (_rad2 / 2 - self.widget.size[0] / 2),
					self.widget.y - (_rad2 / 2 - self.widget.size[0] / 2),
				],
				group="ttv_group",
			)

			# Target ripple.
			Color(
				*self.target_circle_color,
				self.widget.target_ripple_alpha,
				group="ttv_group",
			)
			_rad3 = self.widget.target_ripple_radius
			Ellipse(
				size=[_rad3, _rad3],
				pos=[
					self.widget.x - (_rad3 / 2 - self.widget.size[0] / 2),
					self.widget.y - (_rad3 / 2 - self.widget.size[0] / 2),
				],
				group="ttv_group",
			)

	def stop(self, *args):
		# It needs a better implementation.
		self.anim_ripple.unbind(on_complete=self._repeat_ripple)
		self.description_text_color = [1, 1, 1, 0]
		self.title_text_color = [1, 1, 1, 0]
		anim = Animation(
			d=0.15,
			t="in_cubic",
			**{
				"outer_radius" : 0,
				"target_radius" : 0,
				"target_ripple_radius": 0
			}
		)
		anim.bind(on_complete=self._after_stop)
		anim.start(self.widget)

	def _after_stop(self, *args):
		self.widget.canvas.before.remove_group("ttv_group")
		args[0].stop_all(self.widget)
		elev = getattr(self.widget, "elevation", None)

		if elev:
			self._fix_elev()
		if self.end:
			self.end(self)

		# Don't forget to unbind the function or it'll mess
		# up with other next bindings.
		self.widget.unbind(on_touch_down=self._some_func)

	def _fix_elev(self):
		with self.widget.canvas.before:
			Color(a=self.widget._soft_shadow_a)
			Rectangle(
				texture=self.widget._soft_shadow_texture,
				size=self.widget._soft_shadow_size,
				pos=self.widget._soft_shadow_pos,
			)
			Color(a=self.widget._hard_shadow_a)
			Rectangle(
				texture=self.widget._hard_shadow_texture,
				size=self.widget._hard_shadow_size,
				pos=self.widget._hard_shadow_pos,
			)

			Color(a=1)

	def start(self, *args):
		self._initialize()
		self._animate_outer()

	def _animate_outer(self):
		anim = Animation(
			d=0.2,
			t="out_cubic",
			**{
				"outer_radius" : self.outer_radius,
				"target_radius": self.target_radius
			}
		)

		anim.cancel_all(self.widget)
		anim.bind(on_progress=lambda x, y, z: self._draw_canvas())
		anim.bind(on_complete=self._animate_ripple)
		anim.start(self.widget)
		setattr(self.widget, "target_ripple_radius", self.target_radius)
		setattr(self.widget, "target_ripple_alpha", 1)

	def _animate_ripple(self, *args):
		self.anim_ripple = Animation(
			d=1,
			t="in_cubic",
			target_ripple_radius=self.target_radius + self.ripple_max_dist,
			target_ripple_alpha=0,
		)

		self.anim_ripple.stop_all(self.widget)
		self.anim_ripple.bind(on_progress=lambda x, y, z: self._draw_canvas())
		self.anim_ripple.bind(on_complete=self._repeat_ripple)
		self.anim_ripple.start(self.widget)

	def _repeat_ripple(self, *args):
		setattr(self.widget, "target_ripple_radius", self.target_radius)
		setattr(self.widget, "target_ripple_alpha", 1)
		self._animate_ripple()

	def _some_func(self, wid, touch):
		"""
		This function decides which one to dispatch based on the touch
		position.
		"""

		if self._check_pos_target(touch.pos):
			self.dispatch("on_target_touch")
		elif self._check_pos_outer(touch.pos):
			self.dispatch("on_outer_touch")
		else:
			self.dispatch("on_outside_click")

	def _check_pos_outer(self, pos):
		"""
		Checks if a given `pos` coordinate is within the :attr:`~outer_radius`.
		"""

		cx = ( self.circ_pos[0] + self.outer_radius / 2 )
		cy = ( self.circ_pos[1] + self.outer_radius / 2 )
		r = ( self.outer_radius / 2 )
		h, k = pos

		lhs = (cx - h) ** 2 + (cy - k) ** 2
		rhs = r ** 2

		if lhs <= rhs:
			return True
		return False

	def _check_pos_target(self, pos):
		"""
		Checks if a given `pos` coordinate is within the
		:attr:`~target_radius`.
		"""

		cx = ( self.widget.pos[0] + self.widget.width / 2 )
		cy = ( self.widget.pos[1] + self.widget.height / 2 )
		r = ( self.target_radius / 2 )
		h, k = pos

		lhs = (cx - h) ** 2 + (cy - k) ** 2
		rhs = r ** 2
		
		if lhs <= rhs:
			return True
		return False

	def _ttv_pos(self):
		"""
		Calculates the `pos` value for outer circle and text
		based on the position provided.

		:returns: A tuple containing pos for the circle and text.
		"""

		_rad1 = self.widget.outer_radius
		_center_x = ( self.widget.x - (_rad1 / 2 - self.widget.size[0] / 2) )
		_center_y = ( self.widget.y - (_rad1 / 2 - self.widget.size[0] / 2) )

		if self.widget_position == "left":
			circ_pos = ( _center_x + _rad1 / 3, _center_y )
			title_pos = ( _center_x + _rad1 / 1.4, _center_y + _rad1 / 1.4 )

		elif self.widget_position == "right":
			circ_pos = ( _center_x - _rad1 / 3, _center_y )
			title_pos = ( _center_x - _rad1 / 10, _center_y + _rad1 / 1.4 )

		elif self.widget_position == "top":
			circ_pos = ( _center_x, _center_y - _rad1 / 3 )
			title_pos = ( _center_x + _rad1 / 4, _center_y + _rad1 / 4 )
		
		elif self.widget_position == "bottom":
			circ_pos = ( _center_x, _center_y + _rad1 / 3 )
			title_pos = ( _center_x + _rad1 / 4, _center_y + _rad1 / 1.2 )
		
		# Corner ones need to be at a little smaller distance
		# than edge ones that's why _rad1/4.
		elif self.widget_position == "left_top":
			circ_pos = ( _center_x + _rad1 / 4, _center_y - _rad1 / 4 )
			title_pos = ( _center_x + _rad1 / 2, _center_y + _rad1 / 4 )

		elif self.widget_position == "right_top":
			circ_pos = ( _center_x - _rad1 / 4, _center_y - _rad1 / 4 )
			title_pos = ( _center_x - _rad1 / 10, _center_y + _rad1 / 4 )

		elif self.widget_position == "left_bottom":
			circ_pos = ( _center_x + _rad1 / 4, _center_y + _rad1 / 4 )
			title_pos = ( _center_x + _rad1 / 2, _center_y + _rad1 / 1.2 )

		elif self.widget_position == "right_bottom":
			circ_pos = ( _center_x - _rad1 / 4, _center_y + _rad1 / 4 )
			title_pos = ( _center_x, _center_y + _rad1 / 1.2 )

		else:
			# Center.
			circ_pos = [_center_x, _center_y]

			if self.title_position == "auto":
				raise ValueError(
					"widget_position='center' requires title_position to be set."
				)
			
			elif self.title_position == "left":
				title_pos = ( _center_x + _rad1 / 10, _center_y + _rad1 / 2 )
			
			elif self.title_position == "right":
				title_pos = ( _center_x + _rad1 / 1.6, _center_y + _rad1 / 2 )
			
			elif self.title_position == "top":
				title_pos = ( _center_x + _rad1 / 2.5, _center_y + _rad1 / 1.3 )
			
			elif self.title_position == "bottom":
				title_pos = ( _center_x + _rad1 / 2.5, _center_y + _rad1 / 4 )
			
			elif self.title_position == "left_top":
				title_pos = ( _center_x + _rad1 / 8, _center_y + _rad1 / 1.4 )
			
			elif self.title_position == "right_top":
				title_pos = ( _center_x + _rad1 / 2, _center_y + _rad1 / 1.3 )
			
			elif self.title_position == "left_bottom":
				title_pos = ( _center_x + _rad1 / 8, _center_y + _rad1 / 4 )
			
			elif self.title_position == "right_bottom":
				title_pos = ( _center_x + _rad1 / 2, _center_y + _rad1 / 3.5 )
			else:
				raise ValueError(
					f"'{self.title_position}'"
					f"is not a valid value for title_position"
				)

		self.circ_pos = circ_pos
		return [circ_pos, title_pos]




from kivy.graphics import (
	CanvasBase, Color,
	Ellipse, ScissorPush,
	ScissorPop, RoundedRectangle,
)
from kivy.graphics.stencil_instructions import (
	StencilPop, StencilPush,
	StencilUnUse, StencilUse,
)
from kivy.properties import (
	ListProperty, NumericProperty,
	StringProperty,
)

from kivy.uix.relativelayout import RelativeLayout
from kivy.uix.widget import Widget
from kivy.animation import Animation
from kivy.metrics import dp
from kivy.clock import Clock


class EffectBehavior(Widget):

	#Size Ellipse
	radius_ellipse_default = NumericProperty(dp(10))
	radius_ellipse = NumericProperty(dp(10))

	#Type transition
	transition_in = StringProperty('in_cubic')
	transition_out = StringProperty('out_quad')

	#Duration of transition
	duration_in = NumericProperty(0.3)
	duration_out = NumericProperty(0.2)

	#To know the touch.pos of the widget
	touch_pos = ListProperty([0, 0])

	#Color background_effect
	effect_color = ListProperty([1, 1, 1, 0.5])
	effect_fade_color = ListProperty([1, 1, 1, 0])
	_color_rgba = ListProperty([0, 0, 0, 0])

	#To know if is a `rouded` or `rectangle`...
	type_button = StringProperty('rounded')

	#radius if Rounded
	radius_effect = ListProperty([dp(10), dp(10), dp(10), dp(10)])

	def __init__(self, **kwargs):
		self.register_event_type('on_touch_anim_end')
		super().__init__(**kwargs)
		self.radius_ellipse_default = self.radius_ellipse

		self.bind(touch_pos=self.set_ellipse,
				  radius_ellipse=self.set_ellipse,
				  _color_rgba=self.set_ellipse)
		
		self.bind(pos=self.draw_effect, size=self.draw_effect)

		self.ripple_ellipse = None
		self.ripple_rectangle = None
		self.ripple_col_instruction = None
		self.ripple_pane = None
		Clock.schedule_once(self.set_config)

	def set_config(self,  *args):
		self.ripple_pane = CanvasBase()
		self.canvas.before.add(self.ripple_pane)

	def ripple_show(self, touch):
		Animation.cancel_all(self, 'radius_ellipse', '_color_rgba')
		if isinstance(self, RelativeLayout):
			pos_x, pos_y = self.to_window(*self.pos)
			self.touch_pos = (touch.x-pos_x, touch.y-pos_y)
		else:
			self.touch_pos = (touch.x, touch.y)
		
		self.draw_effect()
		self._color_rgba = self.effect_color
		anim = Animation(
			radius_ellipse=(max(self.size)*2),
			t=self.transition_in,
			_color_rgba=self.effect_color,
			duration=self.duration_in,
		)
		anim.start(self)

	def draw_effect(self, *args):
		if self.ripple_pane == None:
			return None

		self.reset_CanvasBase()

		with self.ripple_pane:
			if self.type_button == 'rounded':
				StencilPush()
				self.ripple_rectangle = RoundedRectangle(
					size=self.size, pos=self.pos,
					radius=self.radius_effect[::-1],
				)
				StencilUse()
				self.ripple_col_instruction = Color(rgba=self._color_rgba)
				self.ripple_ellipse = Ellipse(
					size=(self.radius_ellipse for _ in range(2)),
					pos=(x-self.radius_ellipse/2 for x in self.touch_pos),
				)
				StencilUnUse()
				self.ripple_rectangle = RoundedRectangle(
					size=self.size, pos=self.pos, 
					radius=self.radius_effect[::-1],
				)
				StencilPop()

			elif self.type_button == 'rectangle':
				ScissorPush(pos=self.pos, size=self.size)
				self.ripple_col_instruction = Color(rgba=self._color_rgba)
				self.ripple_ellipse = Ellipse(
					size=(self.radius_ellipse for _ in range(2)),
					pos=(x-self.radius_ellipse/2 for x in self.touch_pos),
				)
				ScissorPop()

	def ripple_fade(self):
		Animation.cancel_all(self, 'radius_ellipse', '_color_rgba')
		anim = Animation(
			radius_ellipse=max(self.size)*2,
			_color_rgba=self.effect_fade_color,
			t=self.transition_out,
			duration=self.duration_out
		)
		anim.bind(on_complete=self.reset_CanvasBase)
		anim.start(self)

	def set_ellipse(self, instance, value):
		if not self.ripple_ellipse:
			return None

		self.ripple_ellipse.size = (self.radius_ellipse for _ in range(2))
		self.ripple_ellipse.pos = (x-self.radius_ellipse/2 for x in self.touch_pos)
		self.ripple_col_instruction.rgba = self._color_rgba

	def reset_CanvasBase(self, *args):
		self.ripple_rectangle = None
		self.ripple_ellipse = None
		self.radius_ellipse = self.radius_ellipse_default
		self.ripple_pane.clear()
		self.dispatch('on_touch_anim_end')
	
	def on_touch_anim_end(self, *args):
		pass





from kivy.uix.layout import Layout
from kivy.properties import (
    NumericProperty, ReferenceListProperty,
    OptionProperty, BoundedNumericProperty,
    VariableListProperty, AliasProperty,
)
from math import sin, cos, pi, radians


class CircularLayout(Layout):

    padding = VariableListProperty([0, 0, 0, 0])
    '''Padding between the layout box and it's children: [padding_left,
    padding_top, padding_right, padding_bottom].

    padding also accepts a two argument form [padding_horizontal,
    padding_vertical] and a one argument form [padding].
    '''

    start_angle = NumericProperty(0)
    '''Angle (in degrees) at which the first widget will be placed.
    Start counting angles from the X axis, going counterclockwise.
    '''

    circle_quota = BoundedNumericProperty(360, min=0, max=360)
    '''Size (in degrees) of the part of the circumference that will actually
    be used to place widgets.
    '''

    direction = OptionProperty("ccw", options=("cw", "ccw"))
    '''Direction of widgets in the circle.
    '''

    outer_radius_hint = NumericProperty(1)
    '''Sets the size of the outer circle. A number greater than 1 will make the
    widgets larger than the actual widget, a number smaller than 1 will leave
    a gap.
    '''

    inner_radius_hint = NumericProperty(0.6)
    '''Sets the size of the inner circle. A number greater than
    :attr:`outer_radius_hint` will cause glitches. The closest it is to
    :attr:`outer_radius_hint`, the smallest will be the widget in the layout.
    '''

    radius_hint = ReferenceListProperty(inner_radius_hint, outer_radius_hint)
    '''Combined :attr:`outer_radius_hint` and :attr:`inner_radius_hint` in a list
    for convenience. See their documentation for more details.
    '''


    def _get_delta_radii(self):
        radius = min(self.width-self.padding[0]-self.padding[2], self.height-self.padding[1]-self.padding[3]) / 2.0
        outer_r = radius * self.outer_radius_hint
        inner_r = radius * self.inner_radius_hint
        return outer_r - inner_r
    
    delta_radii = AliasProperty(_get_delta_radii, None, bind=("radius_hint", "padding", "size"))


    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self.bind(
            start_angle=self._trigger_layout,
            parent=self._trigger_layout,
            children=self._trigger_layout,
            size=self._trigger_layout,
            radius_hint=self._trigger_layout,
            pos=self._trigger_layout
        )

    def do_layout(self, *largs):
        # optimize layout by preventing looking at the same attribute in a loop
        len_children = len(self.children)
        if len_children == 0:
            return None
        
        selfcx = self.center_x
        selfcy = self.center_y
        direction = self.direction

        cquota = radians(self.circle_quota)
        start_angle_r = radians(self.start_angle)
        padding_left = self.padding[0]
        padding_top = self.padding[1]
        padding_right = self.padding[2]
        padding_bottom = self.padding[3]
        padding_x = padding_left + padding_right
        padding_y = padding_top + padding_bottom

        radius = min(self.width-padding_x, self.height-padding_y) / 2.
        outer_r = radius * self.outer_radius_hint
        inner_r = radius * self.inner_radius_hint
        middle_r = radius * sum(self.radius_hint) / 2.
        delta_r = outer_r - inner_r

        # calculate maximum space used by size_hint
        stretch_weight_angle = 0.
        for w in self.children:
            sha = w.size_hint_x
            if sha == None:
                raise ValueError("size_hint_x cannot be None in a CircularLayout")
            else:
                stretch_weight_angle += sha

        sign = 1.0
        angle_offset = start_angle_r
        if direction == 'cw':
            angle_offset = 2 * pi - start_angle_r
            sign = -(1.0)

        for c in reversed(self.children):
            sha = c.size_hint_x
            shs = c.size_hint_y

            angle_quota = cquota / stretch_weight_angle * sha
            angle = angle_offset + (sign * angle_quota / 2)
            angle_offset += sign * angle_quota

            # kived: looking it up, yes. x = cos(angle) * radius + centerx; y = sin(angle) * radius + centery
            ccx = cos(angle) * middle_r + selfcx + padding_left - padding_right
            ccy = sin(angle) * middle_r + selfcy + padding_bottom - padding_top

            c.center_x = ccx
            c.center_y = ccy
            if shs:
                s = delta_r * shs
                c.width = s
                c.height = s





from svg.path.path import Line, CubicBezier
from xml.dom import minidom
from kivy.utils import get_color_from_hex
import os


def do_correction_path(path_filename):
		bar_init = '/' if path_filename.startswith('/') else ''
		new_str = []

		for x in path_filename.split('\\'):
			new_str.extend(x.split(r'/'))

		path = [f'{x}/' for x in new_str[0:-1] if x != '']
		return ''.join([bar_init] + path + [new_str[-1]])

root_path = do_correction_path(os.path.split(__file__)[0])


def rgb_2_dec(color):
	return [channel / 255 for channel in color]


def dec_2_rgb(color):
	return [int(channel * 255) for channel in color]



# working with ratio to get a given svg path coordinate wrt to given widget
"""
x_pos, y_pos: svg path coordinate
wx, wy: widget pos
w, h: widget size
sw, sh: svg size
sf: svg_file, only needed for kivy icon support
"""
# Special support for Kivy svg Icon :)

def x(x_pos, wx, w, sw, sf):

    if "kivy" in sf:
        return ( wx + ( w * (x_pos / 10) / sw) )
    
    return ( wx + (w * x_pos / sw) )


def y(y_pos, wy, h, sh, sf):
    
    if "kivy" in sf:
        return ( wy + (h * ((y_pos / 10)) / sh) )
    
    return ( wy + (h * (sh - y_pos) / sh) )


def point(complex_point: complex, w, h, wx, wy, sw, sh, sf): 
    return [
        x(complex_point.real, wx, w, sw, sf),
        y(complex_point.imag, wy, h, sh, sf)
    ]


def bezier_points(e: CubicBezier, w, h, wx, wy, sw, sh, sf):
    return [
        *point(e.start, w, h, wx, wy, sw, sh, sf),
        *point(e.control1, w, h, wx, wy, sw, sh, sf),
        *point(e.control2, w, h, wx, wy, sw, sh, sf),
        *point(e.end, w, h, wx, wy, sw, sh, sf),
    ]


def line_points(e: Line, w, h, wx, wy, sw, sh, sf):
    return [
        *point(e.start, w, h, wx, wy, sw, sh, sf),
        *point(e.end, w, h, wx, wy, sw, sh, sf),
    ]


# https://stackoverflow.com/a/15399173/8871954
def B0_t(t):
    return (1 - t) ** 3

def B1_t(t):
    return 3 * t * (1 - t) ** 2

def B2_t(t):
    return 3 * t ** 2 * (1 - t)

def B3_t(t):
    return t ** 3


def get_all_points(start, c1, c2, end):
    points = []
    ax, ay = start
    dx, dy = end
    bx, by = c1
    cx, cy = c2

    seg = 1 / 40
    t = 0

    while t <= 1:
        points.extend(
            [
                (B0_t(t) * ax) + (B1_t(t) * bx) + (B2_t(t) * cx) + (B3_t(t) * dx),
                (B0_t(t) * ay) + (B1_t(t) * by) + (B2_t(t) * cy) + (B3_t(t) * dy),
            ]
        )
        t += seg

    return points

def parse_svg(svg_file):
    doc = minidom.parse(svg_file)

    viewbox_string = doc.getElementsByTagName("svg")[0].getAttribute("viewBox")

    sep = "," if "," in viewbox_string else " "
    sw_size = list( map(int, viewbox_string.split(sep)[2:]) )
    
    path_count = 0
    path_strings = []

    for path in doc.getElementsByTagName("path"):
        id_ = path.getAttribute("id") or f"path_{path_count}"
        d = path.getAttribute("d")
        try:
            clr = get_color_from_hex(path.getAttribute("fill")) or [1, 1, 1, 0]
        except ValueError:
            # if color format is different
            clr = [1, 1, 1, 0]

        path_strings.append((d, id_, clr))
        path_count += 1
    
    doc.unlink()
    return [sw_size, path_strings]

def find_center(input_list):
    middle = float(len(input_list)) / 2
    
    if middle % 2 != 0:
        return input_list[int(middle - 0.5)]
    
    return ( (input_list[int(middle)] + input_list[int(middle-1)]) / 2 )





import os
from kivy.clock import Clock
from kivy.core.window import Window
from kivy.uix.widget import Widget
from kivy.animation import Animation
from kivy.uix.floatlayout import FloatLayout
from kivy.graphics import RenderContext, RoundedRectangle

from kivy.properties import (
    AliasProperty, BoundedNumericProperty,
    ColorProperty, ListProperty,
    NumericProperty, VariableListProperty,
    StringProperty, BooleanProperty,
)

from kivy.graphics import (
    Color, Ellipse,
    StencilPop, StencilPush,
    StencilUnUse, StencilUse,
)

from kivygo.utils import root_path, do_correction_path
from kivygo.behaviors.button import ToggleButtonBehavior


class CommonRipple():

    ripple_rad_default = NumericProperty(1)
    """The starting value of the radius of the ripple effect.
    """

    ripple_color = ColorProperty(None)
    """Ripple color in (r, g, b, a) format.
    """

    ripple_alpha = NumericProperty(0.5)
    """Alpha channel values for ripple effect.
    """

    ripple_scale = NumericProperty(None)
    """Ripple effect scale.
    """

    ripple_duration_in_fast = NumericProperty(0.3)
    """Ripple duration when touching to widget.
    """

    ripple_duration_in_slow = NumericProperty(2)
    """Ripple duration when long touching to widget.
    """

    ripple_duration_out = NumericProperty(0.3)
    """The duration of the disappearance of the wave effect.
    """

    ripple_canvas_after = BooleanProperty(True)
    """The ripple effect is drawn above/below the content.
    """

    ripple_func_in = StringProperty("out_quad")
    """Type of animation for ripple in effect.
    """

    ripple_func_out = StringProperty("out_quad")
    """Type of animation for ripple out effect.
    """

    _ripple_rad = NumericProperty()
    _doing_ripple = BooleanProperty(False)
    _finishing_ripple = BooleanProperty(False)
    _fading_out = BooleanProperty(False)
    _no_ripple_effect = BooleanProperty(False)
    _round_rad = ListProperty([0, 0, 0, 0])

    def lay_canvas_instructions(self):
        raise NotImplementedError

    def start_ripple(self) -> None:
        if not self._doing_ripple:
            self._doing_ripple = True
            anim = Animation(
                _ripple_rad=self.finish_rad,
                t="linear",
                duration=self.ripple_duration_in_slow,
            )
            anim.bind(on_complete=self.fade_out)
            anim.start(self)

    def finish_ripple(self) -> None:
        if self._doing_ripple and not self._finishing_ripple:
            self._finishing_ripple = True
            self._doing_ripple = False
            Animation.cancel_all(self, "_ripple_rad")
            anim = Animation(
                _ripple_rad=self.finish_rad,
                t=self.ripple_func_in,
                duration=self.ripple_duration_in_fast,
            )
            anim.bind(on_complete=self.fade_out)
            anim.start(self)

    def fade_out(self, *args) -> None:
        rc = self.ripple_color
        if not self._fading_out:
            self._fading_out = True
            Animation.cancel_all(self, "ripple_color")
            anim = Animation(
                ripple_color=[rc[0], rc[1], rc[2], 0.0],
                t=self.ripple_func_out,
                duration=self.ripple_duration_out,
            )
            anim.bind(on_complete=self.anim_complete)
            anim.start(self)

    def anim_complete(self, *args) -> None:
        self._doing_ripple = False
        self._finishing_ripple = False
        self._fading_out = False

        if not self.ripple_canvas_after:
            canvas = self.canvas.before
        else:
            canvas = self.canvas.after

        canvas.remove_group("circular_ripple_behavior")
        canvas.remove_group("rectangular_ripple_behavior")

    def on_touch_down(self, touch):
        super().on_touch_down(touch)
        if touch.is_mouse_scrolling:
            return False
        
        if not self.collide_point(touch.x, touch.y):
            return False
        
        if not self.disabled:
            self.call_ripple_animation_methods(touch)
            if isinstance(self, ToggleButtonBehavior):
                return super().on_touch_down(touch)
            else:
                return True

    def call_ripple_animation_methods(self, touch) -> None:
        if self._doing_ripple:
            Animation.cancel_all(
                self, "_ripple_rad", "ripple_color", "rect_color"
            )
            self.anim_complete()
        self._ripple_rad = self.ripple_rad_default
        self.ripple_pos = (touch.x, touch.y)

        if self.ripple_color:
            pass
        elif hasattr(self, "theme_cls"):
            self.ripple_color = self.theme_cls.ripple_color
        else:
            # If no theme, set Gray 300.
            self.ripple_color = [
                0.8784313725490196,
                0.8784313725490196,
                0.8784313725490196,
                self.ripple_alpha,
            ]
        self.ripple_color[3] = self.ripple_alpha
        self.lay_canvas_instructions()
        self.finish_rad = max(self.width, self.height) * self.ripple_scale
        self.start_ripple()

    def on_touch_move(self, touch, *args):
        if not self.collide_point(touch.x, touch.y):
            if not self._finishing_ripple and self._doing_ripple:
                self.finish_ripple()

        return super().on_touch_move(touch, *args)

    def on_touch_up(self, touch):
        if self.collide_point(touch.x, touch.y) and self._doing_ripple:
            self.finish_ripple()

        return super().on_touch_up(touch)

    def _set_ellipse(self, instance, value):
        self.ellipse.size = (self._ripple_rad, self._ripple_rad)

    def _set_color(self, instance, value):
        self.col_instruction.a = value[3]


class CircularRippleBehavior(CommonRipple):

    ripple_scale = NumericProperty(1)
    """See :class:`~CommonRipple.ripple_scale`.
    """

    def lay_canvas_instructions(self) -> None:
        if self._no_ripple_effect:
            return None

        with self.canvas.after if self.ripple_canvas_after else self.canvas.before:
            StencilPush(group="circular_ripple_behavior")
            self.stencil = Ellipse(
                size=[n * self.ripple_scale for n in self.size],
                pos=(
                    self.center_x - (self.width * self.ripple_scale) / 2,
                    self.center_y - (self.height * self.ripple_scale) / 2,
                ),
                group="circular_ripple_behavior",
            )
            StencilUse(group="circular_ripple_behavior")
            self.col_instruction = Color(rgba=self.ripple_color)
            self.ellipse = Ellipse(
                size=(self._ripple_rad, self._ripple_rad),
                pos=(
                    self.center_x - self._ripple_rad / 2.0,
                    self.center_y - self._ripple_rad / 2.0,
                ),
                group="circular_ripple_behavior",
            )
            StencilUnUse(group="circular_ripple_behavior")
            Ellipse(
                pos=self.pos, size=self.size,
                group="circular_ripple_behavior"
            )
            StencilPop(group="circular_ripple_behavior")
            self.bind(
                ripple_color=self._set_color,
                _ripple_rad=self._set_ellipse
            )

    def _set_ellipse(self, instance, value):
        super()._set_ellipse(instance, value)
        if self.ellipse.size[0] > self.width * 0.6 and not self._fading_out:
            self.fade_out()

        self.ellipse.pos = (
            self.center_x - self._ripple_rad / 2.0,
            self.center_y - self._ripple_rad / 2.0,
        )


class CommonElevationBehavior(Widget):

    elevation = BoundedNumericProperty(0, min=0, errorvalue=0)
    """Elevation of the widget.
    """

    shadow_radius = VariableListProperty([0], length=4)
    """Radius of the corners of the shadow.
    """

    shadow_softness = NumericProperty(12)
    """Softness of the shadow.
    """

    shadow_softness_size = BoundedNumericProperty(2, min=2)
    """The value of the softness of the shadow.
    """

    shadow_offset = ListProperty((0, 2))
    """Offset of the shadow.
    """

    shadow_color = ColorProperty([0, 0, 0, 0.6])
    """Offset of the shadow.
    """

    _elevation = 0
    _shadow_color = [0.0, 0.0, 0.0, 0.0]


    def _get_widget_pos(self, *args):
        widget_pos = self.to_window(*self.pos)
        # To list, so it can be compared to self.pos directly.
        return list(widget_pos)

    def _set_widget_pos(self, value):
        self.widget_pos = value

    widget_pos = AliasProperty(_get_widget_pos, _set_widget_pos)


    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        with self.canvas.before:
            self.context = RenderContext(use_parent_projection=True)
        with self.context:
            self.rect = RoundedRectangle(pos=self.pos, size=self.size)

        Clock.schedule_once(self.set_shader_string)
        Clock.schedule_once(lambda x: self.on_elevation(self, self.elevation))
        Window.bind(on_draw=self.on_pos)

    def get_shader_string(self):
        shader_string = ""
        for name_file in ["header.frag", "elevation.frag", "main.frag"]:
            path = do_correction_path(os.path.join(root_path, "elevation", name_file))
            with open(path, encoding="utf-8") as file:
                shader_string += f"{file.read()}\n\n"

        return shader_string

    def set_shader_string(self, *args) -> None:
        self.context["shadow_radius"] = list(map(float, self.shadow_radius))
        self.context["shadow_softness"] = float(self.shadow_softness)
        self.context["shadow_color"] = list(map(float, self.shadow_color))[:-1] + [float(self.opacity)]
        self.context["pos"] = list(map(float, self.rect.pos))
        self.context.shader.fs = self.get_shader_string()

    def update_resolution(self) -> None:
        self.context["resolution"] = (*self.rect.size, *self.rect.pos)

    def on_shadow_color(self, instance, value) -> None:
        def on_shadow_color(*args):
            self._shadow_color = list(map(float, value))[:-1] + [
                float(self.opacity) if not self.disabled else 0
            ]
            self.context["shadow_color"] = self._shadow_color

        Clock.schedule_once(on_shadow_color)

    def on_shadow_radius(self, instance, value) -> None:
        def on_shadow_radius(*args):
            if hasattr(self, "context"):
                self.context["shadow_radius"] = list(map(float, value))

        Clock.schedule_once(on_shadow_radius)

    def on_shadow_softness(self, instance, value) -> None:
        def on_shadow_softness(*args):
            if hasattr(self, "context"):
                self.context["shadow_softness"] = float(value)

        Clock.schedule_once(on_shadow_softness)

    def on_elevation(self, instance, value) -> None:
        def on_elevation(*args):
            if hasattr(self, "context"):
                self._elevation = value
                self.hide_elevation(
                    True if (value <= 0 or self.disabled) else False
                )

        Clock.schedule_once(on_elevation)

    def on_shadow_offset(self, instance, value) -> None:
        self.on_size()

    def on_pos(self, *args) -> None:
        if not hasattr(self, "rect"):
            return None

        self.rect.pos = [
            self.widget_pos[0]
            - ((self.rect.size[0] - self.width) / 2)
            - self.shadow_offset[0],
            self.widget_pos[1]
            - ((self.rect.size[1] - self.height) / 2)
            - self.shadow_offset[1],
        ]

        self.context["mouse"] = [self.rect.pos[0], 0.0, 0.0, 0.0]
        self.context["pos"] = list(map(float, self.rect.pos))
        self.update_resolution()

    def on_size(self, *args) -> None:
        if not hasattr(self, "rect"):
            return None

        # If the elevation value is 0, set the canvas size to zero.
        # Because even with a zero elevation value, the shadow is displayed
        # under the widget. This is visible if we change the scale
        # of the widget.
        width = self.size[0] if self.elevation else 0
        height = self.size[1] if self.elevation else 0
        self.rect.size = (
            width
            + (
                self._elevation
                * self.shadow_softness
                / self.shadow_softness_size
            ),
            height
            + (
                self._elevation
                * self.shadow_softness
                / self.shadow_softness_size
            ),
        )

        self.context["mouse"] = [self.rect.pos[0], 0.0, 0.0, 0.0]
        self.context["size"] = list(map(float, self.rect.size))
        self.update_resolution()

    def on_opacity(self, instance, value):
        """
        Adjusts the transparency of the shadow according to the transparency
        of the widget.
        """

        def on_opacity(*args):
            self._shadow_color = list(map(float, self._shadow_color))[:-1] + [
                float(value)
            ]
            self.context["shadow_color"] = self._shadow_color

        super().on_opacity(instance, value)
        Clock.schedule_once(on_opacity)

    def on_radius(self, instance, value) -> None:
        self.shadow_radius = [value[1], value[2], value[0], value[3]]

    def on_disabled(self, instance, value) -> None:
        if value:
            self._elevation = 0
            self.hide_elevation(True)
        else:
            self.hide_elevation(False)

    def hide_elevation(self, hide: bool) -> None:
        if hide:
            self._elevation = -self.elevation
            self._shadow_color = [0.0, 0.0, 0.0, 0.0]
        else:
            self._elevation = self.elevation
            self._shadow_color = self.shadow_color[:-1] + [float(self.opacity)]

        self.on_shadow_color(self, self._shadow_color)
        self.on_size()
        self.on_pos()


class Thumb(CommonElevationBehavior, CircularRippleBehavior, FloatLayout):
    def _set_ellipse(self, instance, value):
        self.ellipse.size = (self._ripple_rad, self._ripple_rad)
        if self.ellipse.size[0] > self.width * 1.5 and not self._fading_out:
            self.fade_out()
        self.ellipse.pos = (
            self.center_x - self._ripple_rad / 2.0,
            self.center_y - self._ripple_rad / 2.0,
        )
        self.stencil.pos = (
            self.center_x - (self.width * self.ripple_scale) / 2,
            self.center_y - (self.height * self.ripple_scale) / 2,
        )



from kivy.uix.widget import Widget
from kivy.graphics.texture import Texture
from kivy.properties import (
    ListProperty, NumericProperty,
    ObjectProperty, ColorProperty,
)
from kivygo.utils import dec_2_rgb
from PIL import Image, ImageDraw, ImageFilter
from kivy.clock import Clock


class NeuGlowCircular:

    glow_radius = NumericProperty(0)
    """
    Radius of the glow

    attr:`glow_radius` is an :class:`~kivy.properties.NumericProperty`
    and defaults to `0`.
    """

    color = ColorProperty([0, 0, 0, 0])
    """
    Color of the glow. It is suggested to set this property to the color of the
    component that the glow is being added to

    attr:`color` is an :class:`~kivy.properties.ColorProperty`
    and defaults to `[0, 0, 0, 0]`.
    """

    behind_color = ColorProperty([0, 0, 0, 0])
    """
    Color of the component behind the glow. If left blank it will result in
    a black ring around your glow, so make sure you set this color properly

    attr:`behind_color` is an :class:`~kivy.properties.ColorProperty`
    and defaults to `[0, 0, 0, 0]`.
    """

    glow_texture = ObjectProperty()
    """
    The property that holds the actual glow texture object

    """

    glow_size = ListProperty([0, 0])
    """
    A list containing the size of the glow texture
    """

    glow_pos = ListProperty([0, 0])
    """
    A list containing the position of the glow texture
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.blank_texture = Texture.create(size=self.size, colorfmt="rgba")
        Clock.schedule_once(self._create_glow)

    def _create_glow(self, *args):
        # Create blank image
        blank_x_size = int(self.size[0] + self.glow_radius * 2)
        blank_y_size = int(self.size[1] + self.glow_radius * 2)
        shadow = Image.new(
            "RGBA",
            (blank_x_size, blank_y_size),
            color=(tuple(dec_2_rgb(self.behind_color))),
        )
        # Convert to drawable image
        blank_draw = ImageDraw.Draw(shadow)
        x0, y0 = (blank_x_size - self.size[0]) / 2.0, (
            blank_y_size - self.size[1]
        ) / 2.0
        x1, y1 = x0 + self.size[0], y0 + self.size[1]
        blank_draw.ellipse(
            [(x0, y0), (x1, y1)],
            fill=tuple(tuple(dec_2_rgb(self.glow_color))),
        )
        texture = Texture.create(size=(blank_x_size, blank_y_size), colorfmt="rgba")
        shadow = shadow.filter(ImageFilter.GaussianBlur(self.glow_radius / 2))
        texture.blit_buffer(shadow.tobytes(), colorfmt="rgba", bufferfmt="ubyte")
        self.glow_texture = texture
        self.glow_size = (blank_x_size, blank_y_size)
        self.glow_pos = self.pos[0] - x0, self.pos[1] - y0

    def _update_glow_pos(self):
        self.glow_pos = self.pos[0] - self.glow_radius, self.pos[1] - self.glow_radius

    def on_size(self, *args):
        self._create_glow()

    def on_pos(self, *args):
        self._update_glow_pos()

    def on_glow_radius(self, *args):
        self._create_glow()





from kivy.clock import Clock
from kivy.graphics import (
	RenderContext, Fbo, Color,
	ClearColor, ClearBuffers,
	Rectangle,
)
from kivy.uix.boxlayout import BoxLayout
from kivy.properties import StringProperty, ObjectProperty


shader_gradient = '''

#ifdef GL_ES
precision mediump float;
#endif

uniform vec2      resolution;           // viewport resolution (in pixels)
uniform float     time;                 // shader playback time (in seconds)

void main(void)
{
	// Normalized pixel coordinates (from 0 to 1)
	vec2 p = gl_FragCoord.xy/resolution.xy;
	
	vec2 q = p - vec2(0.5, 0.5);

	// Time varying pixel color
	vec3 col = 0.7 + 0.2*cos((time)+p.xyx+vec3(0,2,4));
	//vec3 col = mix( vec3(1.0, 0.5, 0.1), vec3(0.2, 0.9, 0.5), p.y);
	
	//col *= length(q);

	gl_FragColor = vec4(col, 1.0);
}
'''



class GradientWidget(BoxLayout):

	fs = StringProperty(None)

	texture = ObjectProperty(None)

	def __init__(self, **kwargs):

		self.canvas = RenderContext(
			use_parent_projection=True,
			use_parent_modelview=True,
			use_parent_frag_modelview=True
		)

		with self.canvas:
			self.fbo = Fbo(size=self.size)
			self.fbo_color = Color(1, 1, 1, 1)
			self.fbo_rect = Rectangle(size=self.size, pos=self.pos)
		
		with self.fbo:
			ClearColor(0, 0, 0, 0)
			ClearBuffers()
		
		super().__init__(**kwargs)

		self.fs = shader_gradient
		Clock.schedule_interval(self.update_glsl, 0)

	def update_glsl(self, *largs):
		self.canvas['time'] = Clock.get_boottime()
		self.canvas['resolution'] = [float(v) for v in self.size]

	def on_fs(self, instance, value):

		shader = self.canvas.shader
		old_value = shader.fs
		shader.fs = value
		if not shader.success:
			shader.fs = old_value
			raise Exception('compilation failed')

	def add_widget(self, *args, **kwargs):
		c = self.canvas
		self.canvas = self.fbo
		super().add_widget(*args, **kwargs)
		self.canvas = c

	def remove_widget(self, *args, **kwargs):
		c = self.canvas
		self.canvas = self.fbo
		super().remove_widget(*args, **kwargs)
		self.canvas = c

	def on_size(self, instance, value):
		self.fbo.size = value
		self.texture = self.fbo.texture
		self.fbo_rect.size = value

	def on_pos(self, instance, value):
		self.fbo_rect.pos = value

	def on_texture(self, instance, value):
		self.fbo_rect.texture = value





from kivy.utils import get_color_from_hex


PALLET_KEY_COLORS = [
	"primary_color",
	"secondary_color",
	"accent_color",
	"background_color",
	"foreground_color",
	"border_color",
	"text_color",
	"heading_color",
	"subheading_color",
	"error_color",
	"success_color",
	"warning_color",
	"info_color",
	"disabled_color",
	"active_color",
	"inactive_color",
	"hover_color",
	"focus_color",
	"selected_color",
	"unselected_color"
]


class Light:
	primary_color = get_color_from_hex("202529")
	secondary_color = get_color_from_hex("fbfbfb")
	accent_color = get_color_from_hex("1fadd2")
	background_color = get_color_from_hex("bf1d15")
	foreground_color = get_color_from_hex("97a1a6")
	border_color = get_color_from_hex("c49a2f")
	text_color = get_color_from_hex("c7cbcb")
	heading_color = get_color_from_hex("f6d49e")
	subheading_color = get_color_from_hex("cfa599")
	error_color = get_color_from_hex("083578")
	success_color = get_color_from_hex("f5f9fa")
	warning_color = get_color_from_hex("16191b")
	info_color = get_color_from_hex("71bca4")
	disabled_color = get_color_from_hex("727a75")
	active_color = get_color_from_hex("127fcf")
	inactive_color = get_color_from_hex("d4bd46")
	hover_color = get_color_from_hex("d1492a")
	focus_color = get_color_from_hex("b8b1ab")
	selected_color = get_color_from_hex("202529")
	unselected_color = get_color_from_hex("fbfbfb")


class Dark:
	primary_color = get_color_from_hex("083578")
	secondary_color = get_color_from_hex("f5f9fa")
	accent_color = get_color_from_hex("16191b")
	background_color = get_color_from_hex("71bca4")
	foreground_color = get_color_from_hex("727a75")
	border_color = get_color_from_hex("127fcf")
	text_color = get_color_from_hex("d4bd46")
	heading_color = get_color_from_hex("d1492a")
	subheading_color = get_color_from_hex("b8b1ab")
	error_color = get_color_from_hex("202529")
	success_color = get_color_from_hex("fbfbfb")
	warning_color = get_color_from_hex("1fadd2")
	info_color = get_color_from_hex("bf1d15")
	disabled_color = get_color_from_hex("97a1a6")
	active_color = get_color_from_hex("c49a2f")
	inactive_color = get_color_from_hex("c7cbcb")
	hover_color = get_color_from_hex("f6d49e")
	focus_color = get_color_from_hex("cfa599")
	selected_color = get_color_from_hex("083578")
	unselected_color = get_color_from_hex("f5f9fa")





from kivy.uix.floatlayout import FloatLayout
from kivygo.behaviors.hover import HoverBehavior
from kivy.lang.builder import Builder
from kivy.properties import ListProperty, NumericProperty


Builder.load_string("""

<ColoredFloatLayout>:
	canvas.before: 
		Color:
			rgba: self.background_color
		RoundedRectangle:
			pos: self.pos
			size: self.size
			radius: self.radius
	canvas.after:
		Color:
			rgba: self.stroke_color
		Line:
			rounded_rectangle: [*self.pos, *self.size, *self.radius]
			width: self.stroke_width

""")


class ColoredFloatLayout(FloatLayout, HoverBehavior):
	background_color = ListProperty([0, 0, 0, 0])
	radius = ListProperty([0, 0, 0, 0])
	stroke_color = ListProperty([0, 0 ,0 ,0])
	stroke_width = NumericProperty(2)





from math import sin, cos
from array import array
peers = {}


def scale(points, length, width, height):
	for h in range(0, length, 2):
		points[h] = points[h] * width
		points[h+1] = points[h+1] * height


def move(points, length, pos0, pos1):
	for i in range(0, length, 2):
		points[i] = points[i] + pos0
		points[i+1] = points[i+1] + pos1


def rotate(points, length, angle, orig0, orig1):
	c = cos(angle)
	s = sin(angle)

	for j in range(0, length, 2):
		points[j] = points[j] - orig0
		points[j+1] = points[j+1] - orig1
		pj = points[j]
		points[j] = pj * c - points[j+1] * s
		points[j+1] = pj * s + points[j+1] * c
		points[j] = points[j] + orig0
		points[j+1] = points[j+1] + orig1


def calc_segboxes(points, polids, ptids, plens, length, bbox, blefts, bbotts, brghts, btops):
	for k in range(0, length, 2):
		k1 = k + 1
		off = plens[polids[k]] * 2 - 2
		if ptids[k] < plens[polids[k]] - 1:
			k2 = k1 + 1
			k3 = k2 + 1
		else:
			k2 = k - off
			k3 = k2 + 1

		blefts[k] = points[k] if points[k] <= points[k2] else points[k2]
		bbotts[k] = points[k1] if points[k1] <= points[k3] else points[k3]
		brghts[k] = points[k] if points[k] >= points[k2] else points[k2]
		btops[k] = points[k1] if points[k1] >= points[k3] else points[k3]

		if blefts[k] < bbox[0]:
			bbox[0] = blefts[k]

		if brghts[k] > bbox[2]:
			bbox[2] = brghts[k]

		if bbotts[k] < bbox[1]:
			bbox[1] = bbotts[k]

		if btops[k] > bbox[3]:
			bbox[3] = btops[k]


def calc_polboxes(points, plens, bbox, blefts, bbotts, brghts, btops):
	strt = 0
	for p in range(len(plens)):
		left = float("inf")
		bottom = float("inf")
		right = 0.
		top = 0.
		for l in range(strt, strt + plens[p] * 2, 2):
			l1 = l + 1

			if points[l] < left:
				left = points[l]

			if points[l] > right:
				right = points[l]

			if points[l1] < bottom:
				bottom = points[l1]

			if points[l1] > top:
				top = points[l1]

		if left < bbox[0]:
			bbox[0] = left

		if right > bbox[2]:
			bbox[2] = right

		if bottom < bbox[1]:
			bbox[1] = bottom

		if top > bbox[3]:
			bbox[3] = top

		blefts[p] = left
		bbotts[p] = bottom
		brghts[p] = right
		btops[p] = top

		strt = strt + plens[p] * 2


def intersection_w(pts, ptids, plens, opens, t_box):
	t_pts = [
		t_box[0], t_box[1], t_box[2], t_box[1],
		t_box[2], t_box[3], t_box[0], t_box[3]
	]
	o = 0
	strt = 0

	for p in range(len(plens)):
		if p in opens:
			o = 2

		for i in range(strt, strt + plens[p] * 2 - o, 2):
			i1 = i + 1
			off = plens[p] * 2 - 2
			if ptids[i] < plens[p] - 1:
				i2 = i1 + 1
				i3 = i2 + 1
			else:
				i2 = i - off
				i3 = i2 + 1

			v10 = pts[i]
			v11 = pts[i1]
			v20 = pts[i2]
			v21 = pts[i3]

			for j in range(0, 8, 2):
				j1 = j + 1
				t_off = 6
				if j < 6:
					j2 = j1 + 1
					j3 = j2 + 1
				else:
					j2 = j - t_off
					j3 = j2 + 1

				v30 = t_pts[j]
				v31 = t_pts[j1]
				v40 = t_pts[j2]
				v41 = t_pts[j3]

				# Segment intersection detection method:
				# If the vertices v1 and v2 are not on opposite sides of the
				# segment v3, v4, or the vertices v3 and v4 are not on
				# opposite sides of the segment v1, v2, there's no
				# intersection.
				if ((v40 - v30) * (v11 - v31) - (v10 - v30) * (v41 - v31) > 0) == \
						((v40 - v30) * (v21 - v31) - (v20 - v30) * (v41 - v31) > 0):
					continue

				elif ((v20 - v10) * (v31 - v11) - (v30 - v10) * (v21 - v11) > 0) == \
						((v20 - v10) * (v41 - v11) - (v40 - v10) * (v21 - v11) > 0):
					continue

				return [p, ptids[i], 0, j/2]

		strt = strt + plens[p] * 2
	return False


def intersection_f(pts, ptids, plens, opens, t_pts, t_polis, t_ptis, t_plens):
	o = 0
	strt = 0
	for p in range(len(plens)):
		if p in opens:
			o = 2

		for i in range(strt, strt + plens[p] * 2 - o, 2):
			i1 = i + 1
			off = plens[p] * 2 - 2
			if ptids[i] < plens[p] - 1:
				i2 = i1 + 1
				i3 = i2 + 1
			else:
				i2 = i - off
				i3 = i2 + 1

			v10 = pts[i]
			v11 = pts[i1]
			v20 = pts[i2]
			v21 = pts[i3]

			t_strt = 0
			for t_p in range(len(t_plens)):
				for j in range(t_strt, t_strt + t_plens[t_p] * 2, 2):
					j1 = j + 1
					t_off = t_plens[t_polis[j]] * 2 - 2
					if t_ptis[j] < t_plens[t_polis[j]] - 1:
						j2 = j1 + 1
						j3 = j2 + 1
					else:
						j2 = j - t_off
						j3 = j2 + 1
					v30 = t_pts[j]
					v31 = t_pts[j1]
					v40 = t_pts[j2]
					v41 = t_pts[j3]

					# Segment intersection detection method:
					# If the vertices v1 and v2 are not on opposite sides of
					# the segment v3, v4, or the vertices v3 and v4 are not
					# on opposite sides of the segment v1, v2, there's no
					# intersection.
					if ((v40 - v30) * (v11 - v31) - (v10 - v30) * (v41 - v31) > 0) == \
							((v40 - v30) * (v21 - v31) - (v20 - v30) * (v41 - v31) > 0):
						continue

					elif ((v20 - v10) * (v31 - v11) - (v30 - v10) * (v21 - v11) > 0) == \
							((v20 - v10) * (v41 - v11) - (v40 - v10) * (v21 - v11) > 0):
						continue

					return [p, ptids[i], t_p, t_ptis[j]]

				t_strt = t_strt + t_plens[t_p] * 2

		strt = strt + plens[p] * 2
	return False


def intersection(pts, ptids, le, plens, opens, lefts, botts, rghts, tops,
				 t_box, t_pts, t_ptis, t_le, t_plens, t_opens, t_lefts,
				 t_botts, t_rghts, t_tops):
	o = 0
	t_o = 0
	strt = 0
	for p in range(len(plens)):
		if p in opens:
			o = 2

		for i in range(strt, strt + plens[p] * 2 - o, 2):
			if rghts[i] < t_box[0]:
				continue

			if lefts[i] > t_box[2]:
				continue

			if tops[i] < t_box[1]:
				continue

			if botts[i] > t_box[3]:
				continue

			i1 = i + 1
			off = plens[p] * 2 - 2
			if ptids[i] < plens[p] - 1:
				i2 = (i1 + 1) % le
				i3 = i2 + 1
			else:
				i2 = i - off
				i3 = i2 + 1

			v10 = pts[i]
			v11 = pts[i1]
			v20 = pts[i2]
			v21 = pts[i3]

			t_strt = 0
			for t_p in range(len(t_plens)):
				if t_p in t_opens:
					t_o = 2
				for j in range(t_strt, t_strt + t_plens[t_p] * 2 - t_o, 2):
					if rghts[i] < t_lefts[j]:
						continue

					if lefts[i] > t_rghts[j]:
						continue

					if tops[i] < t_botts[j]:
						continue

					if botts[i] > t_tops[j]:
						continue

					j1 = j + 1
					t_off = t_plens[t_p] * 2 - 2
					if t_ptis[j] < t_plens[t_p] - 1:
						j2 = (j1+1) % t_le
						j3 = j2 + 1
					else:
						j2 = j - t_off
						j3 = j2 + 1

					v30 = t_pts[j]
					v31 = t_pts[j1]
					v40 = t_pts[j2]
					v41 = t_pts[j3]

					# Segment intersection detection method:
					# If the vertices v1 and v2 are not on opposite sides of
					# the segment v3, v4, or the vertices v3 and v4 are not
					# on opposite sides of the segment v1, v2, there's no
					# intersection.
					if ((v40 - v30) * (v11 - v31) - (v10 - v30) * (v41 - v31) > 0) == \
							((v40 - v30) * (v21 - v31) - (v20 - v30) * (v41 - v31) > 0):
						continue

					elif ((v20 - v10) * (v31 - v11) - (v30 - v10) * (v21 - v11) > 0) == \
							((v20 - v10) * (v41 - v11) - (v40 - v10) * (v21 - v11) > 0):
						continue

					return [p, ptids[i], t_p, t_ptis[j]]

				t_strt = t_strt + t_plens[t_p] * 2

		strt = strt + plens[p] * 2
	return False


def membership(pts, plens, lefts, botts, rghts, tops,
			   t_box, t_pts, t_polis, t_le):
	strt = 0
	for p, pl in enumerate(plens):
		# Preliminary 1: pol's bbox vs widget's bbox.
		if rghts[p] < t_box[0]:
			strt = strt + pl * 2
			continue

		if lefts[p] > t_box[2]:
			strt = strt + pl * 2
			continue

		if tops[p] < t_box[1]:
			strt = strt + pl * 2
			continue

		if botts[p] > t_box[3]:
			strt = strt + pl * 2
			continue

		for k in range(0, t_le, 2):
			x = t_pts[k]
			y = t_pts[k + 1]
			# Preliminary 2: pol's bbox vs widget's points to filter out.
			if rghts[p] < x:
				continue

			if lefts[p] > x:
				continue

			if tops[p] < y:
				continue

			if botts[p] > y:
				continue

			# Point-in-polygon (oddeven) collision detection method:
			# Checking the membership of each poby assuming a ray at 0 angle
			# from that poto infinity (through window right) and counting
			# the number of times that this ray crosses the polygon line.
			# If this number is odd, the pois inside; if it's even, the pois
			# outside.
			c = 0
			j = strt + pl * 2 - 2
			for i in range(strt, strt + pl * 2, 2):
				x1 = pts[j]
				y1 = pts[j+1]
				x2 = pts[i]
				y2 = pts[i+1]
				if (((y2 > y) != (y1 > y))
						and x < (x1 - x2) * (y - y2) / (y1 - y2) + x2):
					c = not c
				j = i

			if c:
				return [p, t_polis[k]]

		strt = strt + pl * 2
	return False


def collide_bounds(rid, wid, frame='bounds', tframe='bounds'):

	try:
		this_box = peers[rid]['bbox']
	except KeyError:
		return False

	try:
		that_box = peers[wid]['bbox']
	except TypeError:
		that_box = wid

	try:
		if this_box[2] < that_box[0]:
			return False
	except IndexError:
		return False

	if this_box[0] > that_box[2]:
		return False

	if this_box[3] < that_box[1]:
		return False

	if this_box[1] > that_box[3]:
		return False

	bounds = peers[rid][frame]
	try:
		tbounds = peers[wid][tframe]
	except TypeError:
		return intersection_w(bounds['points'], bounds['pt_ids'],
							  bounds['pol_lens'], bounds['opens'], that_box)

	if not peers[rid]['seg']:
		return membership(bounds['points'], bounds['pol_lens'],
						  bounds['lefts'], bounds['botts'], bounds['rights'],
						  bounds['tops'], that_box, tbounds['points'],
						  tbounds['pol_ids'],
						  tbounds['length'])

	if peers[wid]['seg']:
		return intersection(bounds['points'], bounds['pt_ids'],
							bounds['length'], bounds['pol_lens'],
							bounds['opens'], bounds['lefts'],
							bounds['botts'], bounds['rights'],
							bounds['tops'], that_box,
							tbounds['points'], tbounds['pt_ids'],
							tbounds['length'], tbounds['pol_lens'],
							tbounds['opens'], tbounds['lefts'],
							tbounds['botts'], tbounds['rights'],
							tbounds['tops'])

	return intersection_f(bounds['points'], bounds['pt_ids'],
						  bounds['pol_lens'], bounds['opens'],
						  tbounds['points'], tbounds['pol_ids'],
						  tbounds['pt_ids'], tbounds['pol_lens'])


def point_in_bounds(x, y, rid, frame='bounds'):
	'''"Oddeven" point-in-polygon method:
	Checking the membership of touch poby assuming a ray at 0 angle
	from that poto infinity (through window right) and counting the
	number of polygon sides that this ray crosses. If this number is
	odd, the pois inside; if it's even, the pois outside.
	'''
	bounds = peers[rid][frame]
	strt = 0
	for r, rang in enumerate(bounds['pol_lens']):
		c = 0
		j = strt + rang * 2 - 2
		for i in range(strt, strt + rang * 2, 2):
			x1, y1 = bounds['points'][j], bounds['points'][j+1]
			x2, y2 = bounds['points'][i], bounds['points'][i+1]
			if (((y2 > y) != (y1 > y)) and
					x < (x1 - x2) * (y - y2) / (y1 - y2) + x2):
				c = not c
			j = i

		if c:
			return c
		strt = strt + rang * 2

	return False


def update_bounds(motion, angle, origin, rid, frame='bounds'):
	'''Updating the elements of the collision detection checks.
	'''
	try:
		bounds = peers[rid][frame]
	except TypeError:
		return None

	if motion:
		move(bounds['points'], bounds['length'], motion[0], motion[1])

	if angle:
		rotate(bounds['points'], bounds['length'], angle, origin[0],
			   origin[1])

	bbox = array('d', [float("inf"), float("inf"), 0., 0.])

	if peers[rid]['seg']:
		calc_segboxes(bounds['points'], bounds['pol_ids'], bounds['pt_ids'],
					  bounds['pol_lens'], bounds['length'], bbox,
					  bounds['lefts'], bounds['botts'], bounds['rights'],
					  bounds['tops'])
	else:
		calc_polboxes(bounds['points'], bounds['pol_lens'], bbox,
					  bounds['lefts'], bounds['botts'], bounds['rights'],
					  bounds['tops'])

	peers[rid]['bbox'] = bbox


def aniupdate_bounds(motion, pos, angle, origin, rid, frame='bounds'):
	'''Updating the elements of the collision detection checks,
	in case of an animation.
	'''
	try:
		bounds = peers[rid][frame]
	except TypeError:
		return None

	if motion:
		bounds['mov_pts'][:] = bounds['sca_pts']
		move(bounds['mov_pts'], bounds['length'], pos[0], pos[1])
		bounds['points'][:] = bounds['mov_pts']

	if angle:
		bounds['points'][:] = bounds['mov_pts']
		rotate(bounds['points'], bounds['length'], angle, origin[0], origin[1])

	bbox = array('d', [float("inf"), float("inf"), 0.0, 0.0])

	if peers[rid]['seg']:
		calc_segboxes(bounds['points'], bounds['pol_ids'], bounds['pt_ids'],
					  bounds['pol_lens'], bounds['length'], bbox,
					  bounds['lefts'], bounds['botts'], bounds['rights'],
					  bounds['tops'])
	else:
		calc_polboxes(bounds['points'], bounds['pol_lens'], bbox,
					  bounds['lefts'], bounds['botts'], bounds['rights'],
					  bounds['tops'])

	peers[rid]['bbox'] = bbox


def resize(width, height, rid):
	for k, frame in peers[rid].items():
		if k == 'bounds':
			bounds = peers[rid]['bounds']
			bounds['points'][:] = bounds['hints']
			scale(bounds['points'], bounds['length'], width, height)
			break
	else:
		for k, frame in peers[rid].items():
			if k != 'bbox' and k != 'vbbox' and k != 'hhits' and k != 'seg' \
					and k != 'friendly':
				frame['points'][:] = frame['hints']
				scale(frame['points'], frame['length'], width, height)


def aniresize(width, height, rid):
	for k, frame in peers[rid].items():
		if k == 'bounds':
			bounds = peers[rid]['bounds']
			bounds['sca_pts'][:] = bounds['hints']
			scale(bounds['sca_pts'], bounds['length'], width, height)
			break
	else:
		for k, frame in peers[rid].items():
			if k != 'bbox' and k != 'seg' and k != '':
				frame['sca_pts'][:] = frame['hints']
				scale(frame['sca_pts'], frame['length'], width, height)


def define_frame(frame, opens, seg_mode, bounds, ani=False):
	for p in range(len(frame)):
		pol = frame[p]
		plen = len(pol)
		array.extend(bounds['pol_lens'], array('i', [plen]))
		for i in range(plen):
			array.extend(bounds['hints'], array('d', [pol[i][0], pol[i][1]]))
			array.extend(bounds['pol_ids'], array('i', [p, p]))
			array.extend(bounds['pt_ids'], array('i', [i, i]))
			bounds['length'] += 2

	if seg_mode:
		length = bounds['length']
	else:
		length = len(bounds['pol_lens'])
	bounds['lefts'] = array('d', [float("inf")] * length)
	bounds['botts'] = array('d', [float("inf")] * length)
	bounds['rights'] = array('d', [0.] * length)
	bounds['tops'] = array('d', [0.] * length)

	if seg_mode:
		bounds['opens'] = array('i', opens)

	if ani:
		bounds['mov_pts'][:] = bounds['sca_pts'][:] = bounds['hints']
	bounds['points'][:] = bounds['hints']


def define_bounds(custom_bounds, open_bounds, segment_mode, rid, pc):
	'''Organising the data from the user's [self.custom_bounds] hints.
	The [pc] parameter is not used. It's here for compatibility with the
	equivalent function in the cython module.
	'''
	frames = {}

	if isinstance(custom_bounds, dict):   # Animation case
		for key, frame in custom_bounds.items():
			bounds = {'hints': array('d'), 'sca_pts': array('d'),
					  'mov_pts': array('d'), 'points': array('d'),
					  'pol_ids': array('i'), 'pt_ids': array('i'),
					  'pol_lens': array('i'), 'length': 0}
			if isinstance(open_bounds, dict):
				opens = array('i', open_bounds[key])
			else:
				opens = array('i', open_bounds)

			define_frame(frame, opens, segment_mode, bounds, ani=True)

			frames[key] = bounds

	elif isinstance(custom_bounds, list):  # Single image case
		bounds = {'hints': array('d'), 'points': array('d'),
				  'pol_ids': array('i'), 'pt_ids': array('i'),
				  'pol_lens': array('i'), 'length': 0}
		define_frame(custom_bounds, open_bounds, segment_mode, bounds)

		frames['bounds'] = bounds

	frames['bbox'] = array('d', [])
	frames['seg'] = segment_mode

	peers[rid] = frames




from kivy.uix.modalview import ModalView
from kivygo.uix.boxlayout import ColoredBoxLayout
from kivy.lang.builder import Builder
from kivy.properties import StringProperty
from kivygo.uix.label import LabelToScroll
from kivygo.uix.icon import ButtonIcon
from kivygo.uix.button import ButtonEffect
from kivygo.uix.anchorlayout import AnchorLayoutButton


Builder.load_string("""

#:import hex kivy.utils.get_color_from_hex

<BoxPopup>:
	box_background_color: [0, 0, 0, 0]
	box_radius:  [dp(20), dp(20), dp(20), dp(20)]
	box_padding: ['0dp', '15dp', '0dp', '0dp']
	border: [0, 0, 0, 0]
	auto_dismiss: False
	overlay_color: [0, 0, 0, 0]
	background_color: [0, 0, 0, 0]

	ColoredBoxLayout:
		orientation: 'vertical'
		padding: root.box_padding
		background_color: root.box_background_color
		radius: root.box_radius
		id: _colored_box


<ConfirmPopup>:
	msg: "É necessário aceitar os termos de uso!"
	title: "Alerta"
	box_background_color: hex("#038c73")
	padding:
		[dp(330), dp(210), dp(330), dp(210)] \
		if app.root.width > dp(1300) and app.root.height > dp(600) \
		else [dp(130), dp(100), dp(130), dp(100)]
	box_padding: [0, 0, 0, 0]
	overlay_color: [0, 0, 0, 0.7]

	BoxLayout:
		size_hint_y: None
		height: self.minimum_height

		ColoredBoxLayout:
			size_hint_y: None
			height: self.minimum_height
			background_color: hex('#ebeef2')
			radius: [dp(20), dp(20), 0, 0]

			AnchorLayout:
				anchor_y: 'center'
				size_hint_y: None
				height: lb_t.height + dp(10)
				padding: [dp(10), 0, 0, 0]

				LabelToScroll:
					text: root.title
					font_size: "30sp"
					bold: True
					halign: 'left'
					valign: 'center'
					color: hex('#e06031')
					id: lb_t

			AnchorLayoutButton:
				size_hint_y: None
				height: lb_t.parent.height

				ButtonIcon:
					size: ['35dp', '35dp']
					source: app.get_icon('close')
					on_release: root.dismiss()

	AnchorLayout:
		anchor_y: 'center'

		LabelToScroll:
			text: root.msg
			font_size: "20sp"

	AnchorLayout:
		anchor_y: 'center'
		anchor_x: 'center'
		size_hint_y: None
		height: 0 if not self.children else self.children[0].height + dp(30)

		ButtonEffect:
			text: "Continuar"
			size_hint: None, None
			size: '120dp', '55dp'
			radius: [dp(15)]
			background_color: hex('#2c2c2c')
			bold: True
			font_size: '17sp'
			on_release: root.dismiss()

""")


class BoxPopup(ModalView):

	def add_widget(self, widget, *args, **kwargs):
		if isinstance(widget, ColoredBoxLayout):
			return super().add_widget(widget, *args, **kwargs)
		return self.ids._colored_box.add_widget(widget, *args, **kwargs)


class ConfirmPopup(BoxPopup):
	
	msg = StringProperty("É necessário aceitar os termos de uso!")
	title = StringProperty("Alerta")





from kivygo.behaviors.button import ButtonBehavior

from kivy.core.text.markup import MarkupLabel as CoreLabel
from kivy.lang import Builder
from kivy.graphics.texture import Texture

from kivy.uix.label import Label
from kivy.properties import (
	NumericProperty, DictProperty,
	StringProperty, ObjectProperty,
	ListProperty, AliasProperty,
)

from kivy.clock import Clock
from kivy.graphics.vertex_instructions import Quad
from kivy.animation import AnimationTransition
from kivy.compat import string_types
from math import pi, cos, sin, radians
from kivy.vector import Vector
from kivygo.transformations import Transformations


Builder.load_string("""

<LabelGradient>:
	canvas.before:
		# draw the gradient below the normal Label Texture
		Color:
			rgba: [1, 1, 1, 1]
		Rectangle:
			texture: self.gradient
			size: self.texture_size
			pos: [ \
			int(self.center_x - self.texture_size[0] / 2.0), \
			int(self.center_y - self.texture_size[1] / 2.0)]
			
<LabelButton>:
	color: [1, 1, 1, 1]

<LabelToScroll>:
	text: 'Section Option'
	padding_x: '15dp'
	size_hint_y: None
	on_size: self.update_content()
	text_size: self.size
	valign: 'center'
	halign: 'center'
	multiline: True

""")


class LabelGradient(Label):

	gradient = ObjectProperty(None)
	bg_color = ListProperty([0, 0, 0, 255])

	def __init__(self, **kwargs):
		super().__init__(**kwargs)

		# bind to texture to trigger use of gradient
		self.bind(texture=self.fix_texture)

	def fix_texture(self, instance, texture):
		if self.gradient == None:
			return None

		# unbind, so we don't loop
		self.unbind(texture=self.fix_texture)

		# The normal Label texture is transparent except for the text itself
		# This code changes the texture to make the text transparent, and everything else
		# gets set to self.bg_color (a property of LabelGradient)
		pixels = list(self.texture.pixels)
		
		for index in range(3, len(pixels)-4, 4):

			if pixels[index] == 0:
				# change transparent pixels to the bg_color
				pixels[index-3:index+1] = self.bg_color
			else:
				# make the text itself transparent
				pixels[index] = 0

		# create a new texture, blit the new pixels, and apply the new texture
		new_texture = Texture.create(size=self.texture.size, colorfmt='rgba')
		new_texture.blit_buffer(bytes(pixels), colorfmt='rgba', bufferfmt='ubyte')
		new_texture.flip_vertical()
		self.texture = new_texture

	
class LabelButton(ButtonBehavior, Label):
	pass


class LabelToScroll(Label):

	n_lines = NumericProperty(0)
	d_height = NumericProperty(0)

	def __init__(self, **kwargs):
		super().__init__(**kwargs)
		Clock.schedule_once(self.update_content, 3)

	def update_content(self, *args):
		kw = self._label.options.copy()
		kw['text'] = self.text
		kw['text_size'] = [self.text_size[0], None]
		lb = CoreLabel(**kw)
		lb.refresh()
		self.height = lb.texture.size[1]


class AnimatedLabel(Label):
	'''duration of the animation of each letter'''
	letter_duration = NumericProperty()

	'''time to wait before starting to animate each letter'''
	letter_offset = NumericProperty()

	'''target text to set to animate'''
	target_text = StringProperty(u'')

	transition_function = ObjectProperty(AnimationTransition.linear)

	'''this function will get the destination coordinates of the letter,
	and the progress for this letter, must return the current
	coordinates, for a Quad to use
	(x3, y3)¨¨¨¨¨¨¨¨¨¨¨¨¨¨¨¨¨¨¨¨(x2, y2)
	       |      /------\      |
	       |     / __--__ \     |
	(x0, y0)____________________(x1, y1)
	'''
	transform = ObjectProperty(Transformations.bouncey)

	_cache = DictProperty({})
	_time = NumericProperty()

	def _get_progress(self):
		duration = (self.letter_offset * len(self.target_text))
		duration += self.letter_duration 
		if not duration:
			return 0

		return (self._time / duration)

	progress = AliasProperty(
		_get_progress,
		bind=['_time', 'target_text', 'letter_duration', 'letter_offset']
	)

	def on_transition_function(self, instance, value):
		if isinstance(value, string_types):
			self.transition_function = getattr(AnimationTransition, value)

	def on_transform(self, instance, value):
		if isinstance(value, string_types):
			self.transform = getattr(Transformations, value)

	def on_target_text(self, instance, value):
		self.markup = True
		self.text = ''.join(
			u'[ref={}]{}[/ref]'.format(i, l)
			for i, l in enumerate(value)
		)

	def cache_text(self, *args):
		self._cache = {}
		for l in self.target_text:
			if l not in self._cache:
				self._cache[l] = l = Label(
					text=l,
					color=self.color,
					font_size=self.font_size,
					font_name=self.font_name)

	def cleanup(self):
		self.quads = []

	def on_texture(self, instance, value):
		self.canvas.clear()
		self.cache_text()
		self.cleanup()
		with self.canvas:
			for l in self.target_text:
				self.create_letter(l)

	def create_letter(self, letter):
		new_quad = Quad(
					points=[0, 0, 0, 0, 0, 0, 0, 0],
					texture=self._cache[letter].texture
		)
		self.quads.append(new_quad)

	def tick(self, dt):
		self._time += dt
		return (
			self._time < self.letter_duration +
			len(self.target_text) * self.letter_offset
		)

	def on__time(self, instance, value):
		if not self.texture:
			print("texture not ready")
			return None
		
		if len(self.refs) != len(self.target_text):
			print("still no refs?")
			return None

		for i, l in enumerate(self.target_text):
			n_value = value - i * self.letter_offset
			if not 0 < n_value < self.letter_duration:
				continue

			self.update_letter(value, i, l)

	def update_letter(self, time, index, letter):
		new_offset = (time - index * self.letter_offset)
		a = self.transition_function(new_offset / self.letter_duration)
		
		# ref can contain multiple rects, but we will always have
		# just one, assuming no letter is cut in half
		coords = list(self.refs[str(index)][0])
		coords[0] += (self.center_x - self.texture_size[0] / 2)
		coords[1] += (self.center_y - self.texture_size[1] / 2)
		coords[2] += (self.center_x - self.texture_size[0] / 2)
		coords[3] += (self.center_y - self.texture_size[1] / 2)

		points = self.transform(coords, a)
		self.quads[index].points = points
		self.quads[index].texture = self._cache[letter].texture

	def animate(self):
		if not self.target_text:
			return None
		
		self._time = 0
		Clock.unschedule(self.tick)
		Clock.schedule_interval(self.tick, 0)


class AnimatedBezierLabel(AnimatedLabel):

	def compute_bezier(self, points, n):
		'''compute nth segment point among segments of the bezier line
		defined by points

		Beware, complexity is quadratic to the number of points
		'''
		# http://en.wikipedia.org/wiki/De_Casteljau%27s_algorithm
		# as the list is in the form of (x1, y1, x2, y2...) iteration is
		# done on each item and the current item (xn or yn) in the list is
		# replaced with a calculation of "xn + x(n+1) - xn" x(n+1) is
		# placed at n+2. each iteration makes the list one item shorter
		_points = points[:]
		n_points = len(_points)
		for i in range(1, n_points):
			for j in range(n_points - 2 * i):
				_points[j] = _points[j] + (_points[j + 2] - _points[j]) * n

		# we got the coordinates of the point in T[0] and T[1]
		return [_points[0], _points[1]]


	def compute_bezier_length(self, *args):
		if not self.points:
			return 0

		a = 0
		l = 0
		v = Vector(self.compute_bezier(self.points, 0))

		while a < 1:
			a += 0.01
			p = self.compute_bezier(self.points, a)
			l += v.distance(p)
			v = Vector(p)
		return l
	
	points = ListProperty([])

	bezier_length = AliasProperty(
		compute_bezier_length,
		cache=True, bind=['points', ]
	)

	def bezier(self, points, progress):
		# XXX for now we don't care about the original position, so
		# kerning is out, along any sane spacing, this is an experiment
		if not self.points:
			return [0, 0, 0, 0, 0, 0, 0, 0]

		d_ratio = 1 / (self.bezier_length or 1)
		x0, y0, x1, y1 = points

		# initial letter offset of the letter
		dx = (x1 + x0) / 2 - (self.center_x - self.texture_size[0] / 2)
		
		# convert it to a progress offset on the bezier curve
		da = (dx * d_ratio)
		progress = (progress - da)
		w = (x1 - x0)
		h = (y1 - y0)
		e = 0.01
		
		b1 = self.compute_bezier(
			self.points,
			max(0, progress - e)
		)

		b2 = self.compute_bezier(
			self.points,
			min(progress + e, 1)
		)

		cx = (b2[0] + b1[0]) / 2
		cy = (b2[1] + b1[1]) / 2
		a = - pi / 2 + radians((Vector(b2) - Vector(b1)).angle((0, 1)))

		if not 0 <= progress < (1 + e):
			a += pi

		return [
			(cx + cos(a - 3 * pi / 4) * w / 2), (cy + sin(a - 3 * pi / 4) * h / 2),
			(cx + cos(a - 1 * pi / 4) * w / 2), (cy + sin(a - 1 * pi / 4) * h / 2),
			(cx + cos(a + 1 * pi / 4) * w / 2), (cy + sin(a + 1 * pi / 4) * h / 2),
			(cx + cos(a + 3 * pi / 4) * w / 2), (cy + sin(a + 3 * pi / 4) * h / 2),
		]







from kivy.lang import Builder
from kivy.uix.widget import Widget
from kivygo.uix.image import ImageWithSVG

from kivy.properties import (
	ColorProperty, ListProperty, 
	NumericProperty, StringProperty,
	ObjectProperty
)
from kivy.metrics import dp
from kivy.core.window import Window


Builder.load_string("""


<ResizableRectangleBord>:
	size_hint: None, None
	size: '8dp', '8dp'
	radius: [sum(self.size)/2] * 4
	mipmap: True
	allow_strech: True
	keep_ratio: False
	
	canvas:
		Clear
		Color:
			rgba: [0]*4 if self.image_source else root.line_color
		RoundedRectangle:
			pos: [ ( self.x - self.outline_width ), ( self.y - self.outline_width ) ]
			size: [ ( self.width + (self.outline_width*2) ), ( self.height + (self.outline_width*2) ) ]
			radius: root.radius
		Color:
			rgba: [0]*4 if self.image_source else root.background_color
		RoundedRectangle:
			pos: self.pos
			size: self.size
			radius: root.radius

	canvas.after:
		Color:
			rgba: self.color if self.image_source else [0]*4
		Rectangle:
			texture: self.texture
			pos: self.pos
			size: self.size


<ResizableLineBord>:
	canvas:
		Color:
			rgba: self.color
		Line:
			width: dp(1)
			points: self.points


<ResizeSelectBehavior>:
	ResizableLineBord: # TOP LINE
		id: top_line
		color: root._top_line_color
		points: [top_left_rect.right, root.top, top_right_rect.x, root.top]

	ResizableLineBord: # BOTTOM LINE
		id: bottom_line
		color: root._bottom_line_color
		points: [bottom_left_rect.right, root.y, bottom_right_rect.x, root.y]

	ResizableLineBord: # LEFT LINE
		id: left_line
		color: root._left_line_color
		points: [root.x, top_left_rect.y, root.x, bottom_left_rect.top]
	
	ResizableLineBord: # RIGHT LINE
		id: right_line
		color: root._right_line_color
		points: [root.right, top_right_rect.y, root.right, bottom_right_rect.y]

	
	ResizableRectangleBord: # TOP LEFT RECTANGLE
		id: top_left_rect
		x: self.parent.x - (self.width/2)
		y: self.parent.top - (self.height/2)
		background_color: root._bord_background_color
		line_color: root._bord_line_color
		image_source: root.bord_icon_source
		size: root.bord_size
		
	ResizableRectangleBord: # BOTTOM LEFT RECTANGLE
		id: bottom_left_rect
		x: self.parent.x-(self.width/2)
		y: self.parent.y-(self.height/2)
		background_color: root._bord_background_color
		line_color: root._bord_line_color
		image_source: root.bord_icon_source
		size: root.bord_size
	
	ResizableRectangleBord: # BOTTOM RIGHT RECTANGLE
		id: bottom_right_rect
		x: self.parent.right-(self.width/2)
		y: self.parent.y-(self.height/2)
		background_color: root._bord_background_color
		line_color: root._bord_line_color
		image_source: root.bord_icon_source
		size: root.bord_size
	
	ResizableRectangleBord:  # TOP RIGHT RECTANGLE
		id: top_right_rect
		x: self.parent.right - (self.width/2)
		y: self.parent.top - (self.height/2)
		background_color: root._bord_background_color
		line_color: root._bord_line_color
		image_source: root.bord_icon_source
		size: root.bord_size

	ResizableRectangleBord:  # TOP CENTER RECTANGLE
		id: top_center_rect
		x: self.parent.x + (self.parent.width/2) - (self.width/2)
		y: self.parent.top - (self.height/2)
		background_color: root._bord_background_color
		line_color: root._bord_line_color
		image_source: root.bord_icon_source
		size: root.bord_size
	
	ResizableRectangleBord:  # LEFT CENTER RECTANGLE
		id: left_center_rect
		x: self.parent.x - (self.width/2)
		y: self.parent.y + (self.parent.height/2) - (self.height/2)
		background_color: root._bord_background_color
		line_color: root._bord_line_color
		image_source: root.bord_icon_source
		size: root.bord_size

	ResizableRectangleBord:  # RIGHT CENTER RECTANGLE
		id: right_center_rect
		x: self.parent.right - (self.width/2)
		y: self.parent.y + (self.parent.height/2) - (self.height/2)
		background_color: root._bord_background_color
		line_color: root._bord_line_color
		image_source: root.bord_icon_source
		size: root.bord_size

	ResizableRectangleBord:  # BOTTOM CENTER RECTANGLE
		id: bottom_center_rect
		x: self.parent.x + (self.parent.width/2) - (self.width/2)
		y: self.parent.y - (self.height/2)
		background_color: root._bord_background_color
		line_color: root._bord_line_color
		image_source: root.bord_icon_source
		size: root.bord_size

""")


class ResizableRectangleBord(ImageWithSVG):
	outline_width = NumericProperty(dp(2))
	radius = ListProperty([0, 0, 0, 0])
	background_color = ColorProperty("#FFFFFF")
	line_color = ColorProperty("#25d2a2")


class ResizableLineBord(Widget):
	color = ColorProperty("red")
	points = ListProperty([0, 0])


class ResizeSelectBehavior(Widget):

	top_line_color = ColorProperty("#25d2a2")
	bottom_line_color = ColorProperty("#25d2a2")
	left_line_color = ColorProperty("#25d2a2")
	right_line_color = ColorProperty("#25d2a2")

	_top_line_color = ColorProperty("#25d2a2")
	_bottom_line_color = ColorProperty("#25d2a2")
	_left_line_color = ColorProperty("#25d2a2")
	_right_line_color = ColorProperty("#25d2a2")
	
	_bord_background_color = ColorProperty("#FFFFFF")
	_bord_line_color = ColorProperty("#25d2a2")

	bord_background_color = ColorProperty("#FFFFFF")
	bord_line_color = ColorProperty("#25d2a2")
	bord_icon_source = StringProperty("")
	bord_size = ListProperty([dp(8), dp(8)])
	line_color = ColorProperty([0, 0, 0, 0])
	line_highlight_color = ColorProperty("#12715e")

	last_touch_pos = (0, 0)

	__resizes = []
	_last_parent = ObjectProperty(None)
	change_pos_varible = StringProperty("pos")
	moved = 0
	touched = 0
	_in_resize = False


	def __init__(self, **kwargs):
		super().__init__(**kwargs)
		self.selected_side = None

		if not ResizeSelectBehavior.__resizes:
			Window.bind(mouse_pos=self.on_mouse_pos)
		ResizeSelectBehavior.__resizes.append(self)

		self.bind(
			on_top_line_color=self._set_default_colors,
			on_bottom_line_color=self._set_default_colors,
			on_left_line_color=self._set_default_colors,
			on_right_line_color=self._set_default_colors,

			on_bord_background_color=self._set_default_colors,
			on_bord_line_color=self._set_default_colors
		)
		self._set_default_colors()
	
	def on_parent(self, *args):
		if self.parent != None:
			self._last_parent = self.parent
		
		if self.parent == None and ResizeSelectBehavior.__resizes:
			if ResizeSelectBehavior.__resizes[0] is self:
				Window.unbind(mouse_pos=self.on_mouse_pos)
				del ResizeSelectBehavior.__resizes[0]
				
				if ResizeSelectBehavior.__resizes:
					Window.bind(mouse_pos=ResizeSelectBehavior.__resizes[0].on_mouse_pos)
			else:
				ResizeSelectBehavior.__resizes.remove(self)


	def _set_default_colors(self, *args):
		if self._in_resize:
			self._top_line_color = self.top_line_color
			self._bottom_line_color = self.bottom_line_color
			self._left_line_color = self.left_line_color
			self._right_line_color = self.right_line_color

			self._bord_background_color = self.bord_background_color
			self._bord_line_color = self.bord_line_color
		else:
			self._top_line_color = [0, 0, 0, 0]
			self._bottom_line_color = [0, 0, 0, 0]
			self._left_line_color = [0, 0, 0, 0]
			self._right_line_color = [0, 0, 0, 0]
			self._bord_background_color = [0, 0, 0, 0]
			self._bord_line_color = [0, 0, 0, 0]


	def on_line_color(self, *args):
		self.top_line_color = self.line_color
		self.bottom_line_color = self.line_color
		self.left_line_color = self.line_color
		self.right_line_color = self.line_color

	def on_mouse_pos(self, window, touch):
		"""
		When the mouse moves, we check the position of the mouse
		and update the cursor accordingly.
		"""
		for wid in ResizeSelectBehavior.__resizes:
			if wid.collide_point(*wid.to_widget(*touch)):
				collision = wid.collides_with_control_points(wid.to_widget(*touch))
				if collision in ["top left", "bottom right"]:
					Window.set_system_cursor("size_nwse")
				elif collision in ["top right", "bottom left"]:
					Window.set_system_cursor("size_nesw")
				elif collision in ["top", "bottom"]:
					Window.set_system_cursor("size_ns")
				elif collision in ["left", "right"]:
					Window.set_system_cursor("size_we")
				else:
					Window.set_system_cursor("size_all")
				return True
		
		Window.set_system_cursor("arrow")

	def collides_with_control_points(self, touch_pos):
		"""
		Returns True if the mouse is over a control point.
		"""
		x, y = touch_pos

		# Checking mouse is on left edge
		if self.x <= x <= self.x + dp(7):

			if self.y <= y <= self.y + dp(7):

				return "bottom left"
			
			elif self.y + dp(7) <= y <= self.y + self.height - dp(7):

				return "left"
			
			else:
				return "top left"

		# Checking mouse is on top edge
		elif self.x + dp(7) <= x <= self.x + self.width - dp(7):

			if self.y <= y <= self.y + dp(7):

				return "bottom"
			
			elif self.y + self.height - dp(7) <= y <= self.y + self.height:

				return "top"
			
			else:
				return False

		# Checking mouse is on right edge
		elif self.x + self.width - dp(7) <= x <= self.x + self.width:

			if self.y <= y <= self.y + dp(7):

				return "bottom right"
			
			elif self.y + dp(7) <= y <= self.y + self.height - dp(7):

				return "right"
			
			else:
				return "top right"

	def on_touch_down(self, touch):
		if not self.collide_point(*touch.pos):
			return False

		print("TOUCH_DOUBLE -=> ", touch.is_double_tap, self.touched)
		
		self.touched += 1
		# if touch.is_double_tap:
		# 	self.touched = 0
		print("TOQUE -=> ", self.touched)

		if self.touched == 2 and self.moved == -1:
			self._in_resize = False
			self._set_default_colors()
			# self.moved = 2
			return True
		
		if self.touched == 5 and self.moved == -1:
			self._in_resize = True
			self._set_default_colors()
			self.moved = 1
			return True

		if self.touched == 1 or touch.is_double_tap:
			self._in_resize = True
			self._set_default_colors()

			if touch.is_double_tap:
				self.moved = 1
			
			return True

		if self.touched > 2:
			if self.touched in [3, 4, 5]:
				self._in_resize = False
				self._set_default_colors()
				return super().on_touch_down(touch)
			
			return True

		self.moved = 0	
		touch.grab(self)
		self.last_touch_pos = touch.pos
		collision = self.collides_with_control_points(touch.pos)
		selected_side = collision

		if collision == "top":
			self._top_line_color = self.line_highlight_color

		elif collision == "bottom":
			self._bottom_line_color = self.line_highlight_color

		elif collision == "left":
			self._left_line_color = self.line_highlight_color

		elif collision == "right":
			self._right_line_color = self.line_highlight_color

		else:
			if collision not in ["bottom right", "bottom left"]:
				self._top_line_color = self.line_highlight_color
				
			if collision not in ["top right", "top left"]:
				self._bottom_line_color = self.line_highlight_color

			if collision not in ["bottom right", "top right"]:
				self._left_line_color = self.line_highlight_color
			
			if collision not in ["bottom left", "top left"]:
				self._right_line_color = self.line_highlight_color

			if collision not in ["top left", "bottom right", "top right", "bottom left"]:
				selected_side = None
		
		self.selected_side = selected_side
		return True
		
	def on_touch_move(self, touch):
		if self.parent == None:
			return None

		if self.touched in [3, 4, 5]:
			self.moved = 2
			return super().on_touch_move(touch)
		
		self.moved = 1
		lx, ly = self.last_touch_pos
		dx = (touch.x - lx)
		dy = (touch.y - ly)

		lpx, lpy = getattr(self, self.change_pos_varible)
		set_pos = lambda x, y: setattr(self, self.change_pos_varible, [x, y])

		if touch.grab_current is self:
			self.last_touch_pos = touch.pos

			if self.selected_side == "top":
				self.height += dy

			elif self.selected_side == "bottom":
				self.height -= dy
				set_pos(lpx, lpy+dy)

			elif self.selected_side == "left":
				self.width -= dx
				set_pos(lpx+dx, lpy)

			elif self.selected_side == "right":
				self.width += dx

			elif self.selected_side == "top left":
				self.width -= dx
				set_pos(lpx+dx, lpy)
				self.height += dy

			elif self.selected_side == "top right":
				self.width += dx
				self.height += dy

			elif self.selected_side == "bottom left":
				self.width -= dx  # OK
				self.height -= dy
				set_pos(lpx+dx, lpy+dy)

			elif self.selected_side == "bottom right":
				self.width += dx
				self.height -= dy
				set_pos(lpx, lpy+dy)

			elif not self.selected_side:
				# Moving
				set_pos(lpx+dx, lpy+dy)
		
		return True

	def on_touch_up(self, touch):
		if self.moved == 1:
			self.touched = 1

		elif self.moved == 2:
			self.touched = 2

		else:
			self.moved = -1
		# 	self.touched = 1

		
		if self.touched in [3, 4, 5]:
			return super().on_touch_up(touch)

		if touch.grab_current is self:
			touch.ungrab(self)
			self._top_line_color = self.top_line_color
			self._bottom_line_color = self.bottom_line_color
			self._left_line_color = self.left_line_color
			self._right_line_color = self.right_line_color
		
		return True




import os, sys, re
from pathlib import Path
from setuptools import find_packages, setup


assert sys.version_info >= (3, 7, 0), "Requires Python 3.7+"
_init_file_path, _init_file_name = os.path.split(__file__)


def get_version():
    version_file = os.path.join(_init_file_path, "kivygo", "__init__.py")
    
    with open(version_file, "rt", encoding="utf-8") as file:
        version_regex = r"(?<=^__version__ = ['\"])[^'\"]+(?=['\"]$)"
        try:
            version = re.findall(version_regex, file.read(), re.M)[0]
            return version
        except IndexError:
            raise ValueError(f"Unable to find version string in {version_file}.")


def glob_paths(pattern):
    out_files = []
    src_path = os.path.join(_init_file_path, "kivygo")

    for root, dirs, files in os.walk(src_path):
        for file in files:
            if file.endswith(pattern):
                filepath = os.path.join(str(Path(*Path(root).parts[1:])), file)
                try:
                    out_files.append(filepath.split(f"kivygo{os.sep}")[1])
                except IndexError:
                    out_files.append(filepath)

    return out_files


if __name__ == "__main__":
    setup(
        name="kivygo",
        version = get_version(),
        packages = find_packages(include=["kivygo", "kivygo.*"]),
        package_dir = {"kivygo": "kivygo"},
        package_data={
            "kivygo": [
                *glob_paths(".png"),
                *glob_paths(".jpg"),
                *glob_paths(".svg"),
                *glob_paths(".ttf"),
                *glob_paths(".kv"),
                *glob_paths(".frag"),
                *glob_paths(".glsl"),
                *glob_paths(".atlas"),
                *glob_paths(".pex"),
                *glob_paths(".bounds"),
            ]
        },
        install_requires=[
            "kivy>=2.0.0", "pillow",
            "opencv-python", "camera4kivy",
            "matplotlib", "pyzbar",
            "requests", "svglib"
        ],
        setup_requires=[],
        python_requires=">=3.7",
)
